/*
 * KCall
 * Copyright (C) 2005-2006 by basyskom GmbH
 *  @author basysKom GmbH <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "kcall.h"

#include <klocale.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <klineedit.h>
#include <kstdaction.h>
#include <kapplication.h>
#include <kconfigdialog.h>

#include <QTimer>
#include <QPixmap>
#include <QCursor>

// #include "kcallcore.h"
// #include "dialdialog.h"
// #include "diallist.h"
// #include "call.h"
// #include "incomingdialog.h"


KCall::KCall() : KSystemTrayIcon()
     , m_isBlinking(false)
     , m_isBlinkIcon(false)
{
    m_blinkTimer = new QTimer(this);
    connect(m_blinkTimer, SIGNAL(timeout()), this, SLOT(slotBlink()));

    setIcon( loadIcon( offlineIcon ));
#if 0
    // paintGui();
    mkcall = new KCallCore();

    kCallGuiRunning = mkcall->getGuiStatus();
    if(kCallGuiRunning)
        kDebug() << "KCallGui is running!" << endl;
    else
        kDebug() << "KCallGui is not running!" << endl;

    connect(mkcall, SIGNAL(sigGuiStatus(bool)),
            this, SLOT(guiStatus(bool)));

    connect(mkcall,SIGNAL(GuiIncomingCall(const QString &,
                                          const QString &,
                                          const QString &)),
            this, SLOT(slotStartIncomingCall(const QString &,
                                             const QString &,
                                             const QString &)));

    connect(mkcall,SIGNAL(registrationChanged(int)),
            this, SLOT(slotRegistrationChanged(int)));

    incomingdlg = new IncomingDialog(this);
    connect(incomingdlg, SIGNAL(accept(QString)), SLOT(acceptCall(QString)));
    connect(incomingdlg, SIGNAL(reject(QString)), SLOT(rejectCall(QString)));
    connect(incomingdlg, SIGNAL(hangup(QString)), SLOT(hangupCall(QString)));
    connect(incomingdlg, SIGNAL(dial()), this, SLOT(slotDialDlg()));
#endif

    //ringTimer = new QTimer;
    //connect(ringTimer, SIGNAL(timeout()), this, SLOT(ringAction()));

    //dcopClient = new DCOPClient();
    //dcopClient->send("kicker","Panel","restart()","");
    //dcopClient->send("kcall", "KCallCore", "registrate()", "");
    //mkcall->registrate();
    //dcopClient->send("kcall","KCallGuiCore","testing()","");
}


KCall::~KCall()
{
    kDebug() << "KCall::~ KCall()" << endl;
#if 0
    delete mkcall;
#endif
}

#if 0
void KCall::mousePressEvent(QMouseEvent *event)
{
    kDebug() << "KCall::mousePressEvent()" << endl;

    if (event->button() == LeftButton)
    {
        this->showDialList( event );
    }
    else if (event->button() == RightButton)
    {
        this->showMainMenu( event );
    }
    else if (event->button() == MidButton)
    {

    }
}


void KCall::paintGui()
{
    kDebug() << "KCall::paintGui()" << endl;

    onlineIcon  = "phone_connect";
    offlineIcon = "phone_disconnect";
    bellIcon    = "phone_bell";

    setBackgroundMode(X11ParentRelative);

    slotRegistrationChanged(KCallCore::NotConnected);
}


void KCall::showDialList(QMouseEvent *pos)
{
    kDebug() << "KCall::showDialList()" << endl;

    KPopupMenu dialListMenu;
    dialListMenu.insertTitle(i18n("Call List"));
    QStringList lastCalls = mkcall->mDialList->getLastCallsAsString();

    QStringList::ConstIterator end( lastCalls.end() );
    for(QStringList::ConstIterator it = lastCalls.begin(); it != end; ++it)
    {
        QString tmpCallTypeIcon = "forward";
        dialListMenu.insertItem(SmallIcon(tmpCallTypeIcon), *it, this, SLOT(slotDialDlg()));
    }

    /*for(int i = 0; i < mkcall->dialListLength; i++)
    {
        Call *tmpCall = mkcall->mDialList->getCall(i);
        if (tmpCall=0)
            DO SOMETHING USEFUL
        QString tmpCallerNr = tmpCall->callerNr;
        int tmpCallType = tmpCall->type;

        QString datetime = "";
        QString tmpCallTypeIcon;
        QDate tmpCallDate = tmpCall->date;
        QTime tmpCallTime = tmpCall->time;

        if(tmpCallDate != QDate::currentDate())
        {
            datetime += tmpCallDate.toString("dd.MM.yyyy");
            datetime += " ";
        }

        datetime += tmpCallTime.toString("hh:mm:ss");

        switch(tmpCallType)
        {
            case 1:
                //Incoming
                tmpCallTypeIcon = "forward";
                break;
            case 2:
                //Outgoing
                tmpCallTypeIcon = "back";
                break;
            default:
                //unknown
                tmpCallTypeIcon = "help";
                break;
        }

        if(tmpCall->type > 0)
        {
            QString tmpCallInfo = tmpCallerNr+" ("+datetime+")";
            kDebug() << tmpCallInfo << endl;
            dialListMenu->insertItem(SmallIcon(tmpCallTypeIcon), tmpCallInfo, this, SLOT(slotDialDlg()));
        }else
        {
            i = mkcall->dialListLength;
        }
    }*/
    dialListMenu.popup( pos->globalPos() );
}


void KCall::showMainMenu(QMouseEvent *pos)
{
    kDebug() << "KCall::showMainMenu()" << endl;

    KPopupMenu* mainMenu = new KPopupMenu(this);

    //contextMenuAboutToShow( mainMenu );
    int buttonIdmain[10];
    buttonIdmain[0] = mainMenu->insertItem( SmallIcon( "kcall" ), i18n("&Open KCall"),
                                            this, SLOT( slotOpenKCall() ) );
    mainMenu->insertSeparator();
    buttonIdmain[1] = mainMenu->insertItem( SmallIcon( "configure" ), i18n( "&Configure KCall..." ),
                                            this, SLOT( slotPreferences() ) );
    mainMenu->insertSeparator();
    buttonIdmain[2] = mainMenu->insertItem( i18n( "Dial a Phonenumber" ),
                                            this, SLOT( slotDialDlg() ) );
    buttonIdmain[3] = mainMenu->insertItem( regButtonText ,
                                            this, SLOT( slotUnRegistrate() ) );
    buttonIdmain[4] = mainMenu->insertItem( SmallIcon("exit"),i18n( "Quit" ),
                                            kapp, SLOT( quit() ) );

    mainMenu->popup( pos->globalPos() );
}
#endif

void KCall::startBlink( const QPixmap& iconOn, const QPixmap& iconOff )
{
    m_blinkIconOn = iconOn;
    m_blinkIconOff = iconOff;
    if ( m_blinkTimer->isActive() == true )
    { m_blinkTimer->stop(); }

    m_isBlinkIcon = true;
    m_isBlinking = true;
    m_blinkTimer->start( 1000 );
}

void KCall::stopBlink()
{
    if (m_blinkTimer->isActive() )
        m_blinkTimer->stop();

    m_isBlinkIcon = false;
    m_isBlinking = false;
    m_blinkTimer->stop();
}

void KCall::slotBlink()
{
    setIcon( m_isBlinkIcon ? m_blinkIconOn : m_blinkIconOff );
    m_isBlinkIcon = !m_isBlinkIcon;
}

#if 0
void KCall::slotOpenKCall()
{
    if (!kCallGuiRunning)
    {
        QString error;
        KApplication::startServiceByDesktopName(
                "kcall",
                QString::null, &error );
        if (!error.isEmpty())
            KMessageBox::error(0, error);
     }
     else
     {
        /// TODO: bring kcall to front
     }
}
#endif


void KCall::slotAbout()
{
    KMessageBox::information(0, i18n("This is an about box"));
}


void KCall::slotHelp()
{
    KMessageBox::information(0, i18n("This is a help box"));
}


void KCall::slotPreferences()
{
    kDebug() << "KCall::slotPreferences()" << endl;
#if 0
    if (!kCallGuiRunning)
    {
       if(KConfigDialog::showDialog("settings"))
         return;
       KConfigDialog *dialog = new KConfigDialog(this, "settings",
            KCallSipSettings::self(), KDialogBase::IconList);
       dialog->addPage(new SIPPrefs(), KCallSipSettings::self(), i18n("SIP"), "kcall");
       //dialog->addPage(new General(0, "General"), i18n("General") );
       //dialog->addPage(new Appearance(0, "Style"), i18n("Appearance") );
       connect(dialog, SIGNAL(settingsChanged()), this, SLOT(slotSettingsChanged()));
       dialog->show();
    }
    else
    {
        kDebug() << "dcop" << endl;
        mkcall->dcopClient->send("kcall", "KCallGuiCore", "showSettingsDialog()", "");
        //KMessageBox::information(0, i18n("This is not implemented yet."));
    }
#endif
}

void KCall::slotSettingsChanged()
{
#if 0
    mkcall->settingsChanged();
#endif
}

void KCall::slotDialDlg()
{
    kDebug() << "KCall::slotDialDlg()" << endl;
#if 0
    DialDialog dialdlg;
    /*if(num.isEmpty())
    {
        dialdlg->nrInput->setText(num);
    }
    */

    bool res;
    res = dialdlg.exec();

    // @FIXME Mike: what is this about? why is your name within the code?
    if(res){
        QString callId = mkcall->dial(dialdlg.nrInput->text());
        QString caller = "Mike Hauth";
        QString subject = "Hallo Welt!";
        slotStartIncomingCall(callId, caller, subject);
    }
#endif
}


void KCall::slotStartIncomingCall(const QString & callid,
                                  const QString & caller,
                                  const QString & subject)
{
    kDebug() << "KCall::slotStartIncoming()" << endl;

    setIcon( loadIcon( bellIcon ));
#if 0
    connect(mkcall,SIGNAL(callEnded(const QString &)), this, SLOT(slotStopIncomingCall(const QString &)));

    setPixmap( loadIcon( bellIcon ));

    if( kCallGuiRunning )
    {
        kDebug() << "StartIncoming:KCallGui is running!" << endl;

        QByteArray  callData;

        QDataStream senddataA(callData, IO_WriteOnly);
        senddataA << callid << caller << subject << 1;

        mkcall->dcopClient->send("kcall", "KCallGuiCore", "newCall(QString, QString, QString, int)", callData);

    }
    else
    {
        kDebug() << "StartIncoming:KCallGui is not running!" << endl;

        incomingdlg->newCall(callid);
        incomingdlg->show();
    }
    //KNotifyClient::Instance();
    //KNotifyClient::event("incoming-call");
    //KNotifyClient::event(winId(), "incoming-call", "Anruf");
#endif
}

void KCall::slotStopIncomingCall(const QString & callid)
{
    kDebug() << "KCall::slotStopIncomingCall()" << endl;
    setIcon( loadIcon( onlineIcon) );
#if 0
    if( kCallGuiRunning )
    {
        kDebug() << "StopIncoming:KCallGui is running!" << endl;

        mkcall->dcopClient->send("kcall", "KCallGuiCore", "cancelCall(QString)", callid);
        setPixmap( loadIcon( onlineIcon) );
    }
    else
    {
        kDebug() << "StopIncoming:KCallGui is not running!" << endl;
        if(incomingdlg){
            incomingdlg->cancelCall(callid);
            setPixmap( loadIcon( onlineIcon ) );
        }
    }
#endif
}


void KCall::slotRegistrationChanged(int newstatus)
{
    kDebug() << "KCall:slotRegistrationChanged() " << newstatus << endl;

    setIcon( loadIcon( onlineIcon ));
#if 0
    switch (newstatus)
    {
        case KCallCore::Connecting:
            startBlink( loadIcon(onlineIcon), loadIcon(offlineIcon) );
            break;
        case KCallCore::Connected:
            stopBlink();
            setPixmap( loadIcon( onlineIcon ));
            regButtonText = i18n("Disconnect");
            break;
        case KCallCore::Disconnecting:
            startBlink( loadIcon(onlineIcon), loadIcon(offlineIcon) );
            break;
        case KCallCore::NotConnected:
        default:
            stopBlink();
            setPixmap( loadIcon( offlineIcon ));
            regButtonText = i18n("Connect to Server");
    }
#endif
}

void KCall::slotUnRegistrate()
{
#if 0
    kDebug() << "KCall::slotUnRegistrate()" << endl;
    if(mkcall->getRegistration())
    {
        mkcall->unregistrate();
        //dcopClient->send("kcall", "KCallCore", "unregistrate()", "");
    }else
    {
        mkcall->registrate();
        //dcopClient->send("kcall", "KCallCore", "registrate()", "");
    }
#endif
}

void KCall::acceptCall(const QString & callId)
{
    kDebug() << "KCall::acceptCall()" << endl;
#if 0
    mkcall->acceptCall( callId );
#endif
}

void KCall::rejectCall(const QString & callId)
{
    kDebug() << "KCall::rejectCall()" << endl;
#if 0
    mkcall->rejectCall( callId );
#endif
}

void KCall::hangupCall(const QString & callId)
{
    kDebug() << "KCall::hangupCall()" << endl;
#if 0
    mkcall->hangUp( callId );
#endif
}

void KCall::slotQuit()
{
    kDebug() << "KCall::slotQuit()" << endl;

    // ### earth to maintainer: calling quit is extremely bad in an applet
    // because the applet could run in-process and then kicker will be quit!!
    kapp->exit();
}

void KCall::guiStatus(bool s)
{
    kDebug() << "KCall::guiStatus()" << endl;
#if 0
    kCallGuiRunning = s;
#endif
}

#include "kcall.moc"
