/*
 * KCall
 * Copyright (C) 2004-2006 by basyskom GmbH
 *  @author basysKom GmbH <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef KCALLCORE_H
#define KCALLCORE_H

#include <dcopclient.h>
#include <dcopobject.h>

#include <qobject.h>
#include <qstring.h>
#include <qmap.h>
#if QT_VERSION >= 300
#include <qptrstack.h>
#else
#include <qstack.h>
#endif
#include <kconfig.h>
#include "diallist.h"

class CallAudio;
class SipClient;
class SipUser;
class SipCall;
class SipRegister;
class QTimer;
class KSipAuthentication;

class CallManager;

class KCallCore :  public QObject, public DCOPObject
{
    K_DCOP
    Q_OBJECT

    public:
        KCallCore();
        virtual ~KCallCore();

        void readGeneralSettings();
        void readAudioSettings();
        void readClientSettings();
        void resetAudio();
        bool getGuiStatus();
        SipRegister* getSipRegister();

        DialList* mDialList;

        void restartKicker();
        DCOPClient *dcopClient;

        enum RegisterState {
            NotConnected,
            Connecting,
            Connected,
            Disconnecting
            };

    protected:
        void cleanupCallManagers();
        bool registrateAtGui(bool state);

        CallAudio *audio;
        SipClient *client;
        SipUser *user;
        SipRegister *reg;
        QTimer *quitTimer;
        QTimer *stun_timer;
        QTimer *reg_timer;

        bool b_registered;

        /** maps call ids to CallManagers */
        QMap<QString, CallManager*> callManagerMap;
#if QT_VERSION >= 300
        QPtrStack<CallManager> readyForDeleteManagers;
#else
        QStack<CallManager> readyForDeleteManagers;
#endif

        KSipAuthentication* sipauthentication;

        QString codec;
        QString userUri;
        bool useStun;

        QTimer *ringTimer;

        QString username;
        QString password;

    k_dcop:

        void settingsChanged();
        void identityChanged();

        void registrate();
        void unregistrate();

        QString dial(const QString& num);
        void hangUp(const QString& callid);
        void acceptCall(const QString& callid);
        void rejectCall(const QString& callid);

        bool getRegistration();
        void quit();
        void guiStatusReport(bool status);

        void writeComment();//just for experiment

    protected slots:

        void timerTick();
        void stun_timerTick();

        void updateRegistrationStatus();
        void incomingCall( SipCall *call, const QString & body );

        void deleteManager(const QString& callid);
        void checkIfRegistered();

    protected slots:

        void deleteManagerDelayed();
        void startRing();
        void stopRing();
        void ringAction();

    signals:
        void GuiIncomingCall( const QString &, const QString &,
                              const QString & );
        void callEnded( const QString & );
        void registrationChanged( int );
        void sigGuiStatus( bool );

        void Something();   //just for experiment

};
#endif
