/*
 * KCall VoIP/CTI telephony
 * Copyright (C) 2006-2007 by basyskom GmbH
 *  @author Mike Hauth <info@basyskom.de>
 *  @author Tobias Hunger <info@basyskom.de>
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _HISTORYMODEL_H_
#define _HISTORYMODEL_H_

#include <QObject>
#include <QList>
#include <QAbstractTableModel>

#include "callinfo.h"

class KCallCore;
class HistoryModelPrivate;

/**
 *  List of Past Calls
 */

class HistoryModel : public QAbstractTableModel
{
        Q_OBJECT
    public:
        explicit HistoryModel(KCallCore * const core, QObject * parent = 0);
        ~HistoryModel();

        // QtAbstractTableModel implementation:
        int rowCount(const QModelIndex & parent = QModelIndex()) const;
        int columnCount(const QModelIndex & parent = QModelIndex()) const;
        QVariant data(const QModelIndex & index, int role = Qt::DisplayRole) const;
        QVariant headerData (int section, Qt::Orientation orientation,
                             int role = Qt::DisplayRole ) const;

        // Custom methods:
        /**
         * @brief Set the call's subject.
         * @param row The row number to change.
         * @param subject The subject to set.
         */
        void setSubject(const int row, const QString & subject);

        enum ColumnNames
        {
            CALL_ID = 0,
            CALLER_NR,
            CALLER_NAME,
            SUBJECT,
            STATUS,
            DATE,
            TIME,
            DURATION,
            COLUMN_MAX
        };

        // Slots (for D-Bus):
    public slots:
        /** @brief Called when the Decibel Log service is cleared. */
        void doLogCleared();
        /** @brief Called whenever a new entry is added to the Decibel Log
         *         service. */
        void doEntryAdded(const uint);

    private:
        HistoryModelPrivate * const d;
};

#endif
