/*
 * Copyright (C) 2007 by basyskom GmbH
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <QAction>
#include <QVBoxLayout>
#include <QClipboard>

#include <kaboutdata.h>
#include <ktoolbar.h>
#include <kdialog.h>
#include <kactioncollection.h>
#include <kaction.h>
#include <kxmlguiclient.h>
#include <kcmdlineargs.h>
#include <kiconloader.h>

#include <Decibel/Types>

#include "kcallview.h"
#include "decibeldialer.h"
#include "decibelchannelhandler.h"
#include "kcallcore.h"
#include "historymodel.h"
#include "callmodel.h"
#include "dtmfinput.h"

KCallCore::KCallCore( KXMLGUIClient *client, bool readWrite, QWidget *parent,
                  const QString & /* file */, const char *name ) :
    m_widget(new QWidget(parent)),
    m_statusBar( 0 ),
    m_readWrite( readWrite ),
    m_modified( false ),
    m_isPart(false),
    m_actionSave(0),
    m_addressBook(KABC::StdAddressBook::self( true )),
    m_decibelDialer(new DecibelDialer(this)),
    m_decibelChannelHandler(new DecibelChannelHandler(this)),
    m_historyModel(new HistoryModel(this, this)),
    m_callModel(new CallModel(this, this)),
    m_dtmfInput(new DTMFInput(this, 0)),
    m_dtmfInputDialog(0),
    m_guiClient( client )
{
    Decibel::registerTypes();
    setObjectName( name );

    m_isPart = (strcmp(parent->metaObject()->className(), "KCallMainWindow") );

    //Pixmaps for states
    m_statusPix[int(INCOMING)] = SmallIcon("go-next");
    m_statusPix[int(OUTGOING)] = SmallIcon("go-previous");
    m_statusPix[int(REJECTED)] = SmallIcon("rejected");
    m_statusPix[int(HANGUP)] = SmallIcon("hangedup");
    m_statusPix[int(CANCELED)] = SmallIcon("canceled");
    m_statusPix[int(ACTIVE)] = SmallIcon("active");

    connect(m_decibelChannelHandler,
            SIGNAL(newChannel(QtTapioca::Connection * const,
                              QtTapioca::StreamChannel * const, const bool)),
            m_callModel,
            SLOT(addChannel(QtTapioca::Connection * const,
                            QtTapioca::StreamChannel * const,
                            bool)));

    initGUI();
}

KCallCore::~KCallCore()
{ delete m_dtmfInput; }

bool KCallCore::newCall(const QString& /* number */)
{
    // TODO: external api
    return false;
}

KAboutData *KCallCore::createAboutData()
{
    KAboutData *about = new KAboutData( "kcall", 0, ki18n( "KCall" ),
                                        "0.7.0",
                                        ki18n( "The KCall phone application"),
                                        KAboutData::License_GPL_V2,
                                        ki18n( "(c) 2005, Mike Hauth" ) );

    about->addAuthor( ki18n("Eva Brucherseifer"), ki18n("Developer"), "eva.brucherseifer@basyskom.de" );
    about->addAuthor( ki18n("Mike Hauth"), ki18n("Initial version"), "mike@mike-hauth.de" );
    about->addAuthor( ki18n("Tobias Hunger"), ki18n("Developer"), "tobias.hunger@basyskom.de" );

    return about;
}

void KCallCore::setStatusBar( KStatusBar *statusBar )
{
    m_statusBar = statusBar;
}

KStatusBar *KCallCore::statusBar() const
{
    return m_statusBar;
}


QWidget *KCallCore::widget() const
{
    return m_widget;
}

void KCallCore::setModified()
{
    setModified( true );
}

void KCallCore::setModified( bool modified )
{
    m_modified = modified;
    m_actionSave->setEnabled( m_modified );

//    mSearchManager->reload();
}

bool KCallCore::modified() const
{
    return m_modified;
}

void KCallCore::restoreSettings()
{
    /*
    bool state = KABPrefs::instance()->jumpButtonBarVisible();
    mActionJumpBar->setChecked( state );
    setJumpButtonBarVisible( state );

    state = KABPrefs::instance()->detailsPageVisible();
    mActionDetails->setChecked( state );
    setDetailsVisible( state );

    mViewManager->restoreSettings();
    mExtensionManager->restoreSettings();

    updateIncSearchWidget();
    mIncSearchWidget->setCurrentItem( KABPrefs::instance()->currentIncSearchField() );

    QList<int> splitterSize = KABPrefs::instance()->extensionsSplitter();
    if ( splitterSize.count() == 0 ) {
        splitterSize.append( mDetailsSplitter->height() / 2 );
        splitterSize.append( mDetailsSplitter->height() / 2 );
    }
    mExtensionBarSplitter->setSizes( splitterSize );

    splitterSize = KABPrefs::instance()->detailsSplitter();
    if ( splitterSize.count() == 0 ) {
        splitterSize.append( 360 );
        splitterSize.append( 260 );
    }
    mDetailsSplitter->setSizes( splitterSize );
*/
}

void KCallCore::saveSettings()
{
    /*
    KABPrefs::instance()->setJumpButtonBarVisible( mActionJumpBar->isChecked() );
    KABPrefs::instance()->setDetailsPageVisible( mActionDetails->isChecked() );

    KABPrefs::instance()->setExtensionsSplitter( mExtensionBarSplitter->sizes() );
    KABPrefs::instance()->setDetailsSplitter( mDetailsSplitter->sizes() );

    mExtensionManager->saveSettings();
    mViewManager->saveSettings();

    KABPrefs::instance()->setCurrentIncSearchField( mIncSearchWidget->currentItem() );
    */
}

KConfig *KCallCore::config() const
{
//    return KABPrefs::instance()->config();
    return new KConfig();
}

KActionCollection *KCallCore::actionCollection() const
{
    return guiClient()->actionCollection();
}



bool KCallCore::queryClose()
{
    saveSettings();
//    KABPrefs::instance()->writeConfig();

    return true;
}

void KCallCore::initGUI()
{
    // KCallGui:
    QVBoxLayout *topLayout = new QVBoxLayout( m_widget );
    topLayout->setSpacing( 0 );
    topLayout->setMargin( 0 );

    KCallView* call_view = new KCallView(this,  m_widget);
    call_view->setObjectName("call view");
    topLayout->addWidget(call_view);

    connect(m_addressBook, SIGNAL(addressBookChanged(AddressBook*)),
            call_view, SLOT(updatePhonebookView()));
    connect(call_view, SIGNAL(dial(const QString &)),
            m_decibelDialer, SLOT(dial(const QString &)));
    connect(call_view, SIGNAL(hangup(const QString &)),
            m_callModel, SLOT(hangupCall(const QString &)));
    connect(call_view, SIGNAL(accept(const QString &)),
            m_callModel, SLOT(acceptCall(const QString &)));
    connect(call_view, SIGNAL(reject(const QString &)),
            m_callModel, SLOT(hangupCall(const QString &)));
    connect(call_view, SIGNAL(activeChannelSelected(const QString &)),
            m_dtmfInput, SLOT(switchChannel(const QString &)));

    connect(m_dtmfInput, SIGNAL(sendDTMF(const QString &, const QChar)),
            m_callModel, SLOT(sendDTMF(const QString &, const QChar)));

    //connect( mDetails, SIGNAL( addressClicked( const QString&) ),
    //         this, SLOT( showContactsAddress( const QString& ) ) );

    initActions();
}

void KCallCore::initActions()
{
    connect( QApplication::clipboard(), SIGNAL( dataChanged() ),
             SLOT( clipboardDataChanged() ) );

    QAction * action(0);

    action = new KAction(this);
    action->setText(i18n("send DTMF tones"));
    action->setShortcut(Qt::CTRL + Qt::Key_D);
    connect(action, SIGNAL(triggered(bool)),
            this, SLOT(showDTMFInput()));
    actionCollection()->addAction("dtmf_input", action);
    kDebug() << "Action \"dtmf_input\" set up." << endl;

  // file menu
/*    mActionMail = coll->addAction( "file_mail" );
    mActionMail->setIcon( KIcon("mail-send") );
    mActionMail->setText( i18n( "&Send Email to Contact..." ) );
    connect(mActionMail, SIGNAL(triggered(bool)), SLOT( sendMail() ));
    action = KStandardAction::print( this, SLOT( print() ), actionCollection() );
    mActionMail->setWhatsThis( i18n( "Send a mail to all selected contacts." ) );
    action->setWhatsThis( i18n( "Print a special number of contacts." ) );

    mActionSave = KStandardAction::save( this, SLOT( save() ), actionCollection() );
    actionCollection()->addAction( "file_sync", mActionSave );
    mActionSave->setWhatsThis( i18n( "Save all changes of the address book to the storage backend." ) );
    */

/*    mActionEditAddressee = coll->addAction( "file_properties" );
    mActionEditAddressee->setIcon( KIcon("document-properties") );
    mActionEditAddressee->setText( i18n( "&Edit Contact..." ) );
    connect(mActionEditAddressee, SIGNAL(triggered(bool) ), SLOT( editContact() ));
    mActionEditAddressee->setWhatsThis( i18n( "Edit a contact<p>You will be presented with a dialog where you can change all data about a person, including addresses and phone numbers." ) );
*/


  // settings menu
    if ( m_isPart )
    {
        action = actionCollection()->addAction("kcall_configure");
        action->setIcon(KIcon("configure"));
        action->setText(i18n("&Configure KCall..."));
        connect(action, SIGNAL(triggered(bool) ), SLOT( configure() ));
    } else {
        action = KStandardAction::preferences( this, SLOT( configure() ), actionCollection() );
    }
    action->setWhatsThis( i18n( "You will be presented with a dialog, that offers you all possibilities to configure KCall." ) );

  // misc
/*    QAction *clearLocation = coll->addAction( "clear_search" );
    clearLocation->setIcon( KIcon(QApplication::isRightToLeft() ? "edit-clear-locationbar-rtl" : "edit-clear-locationbar-ltr") );
    clearLocation->setText( i18n( "Clear Search Bar" ) );
    connect(clearLocation, SIGNAL(triggered(bool) ), SLOT( slotClearSearchBar() ));
    clearLocation->setShortcut(QKeySequence(Qt::CTRL+Qt::Key_L));
    clearLocation->setWhatsThis( i18n( "Clear Search Bar<p>"
            "Clears the content of the quick search bar." ) );
*/
//    clipboardDataChanged();
}



bool KCallCore::handleCommandLine()
{
    bool doneSomething = false;

    // @todo: Add optionhandling here.

    return doneSomething;
}

KABC::AddressBook * const KCallCore::addressBook() const
{ return m_addressBook; }

KABC::Addressee KCallCore::getPersonFromNumber(QString phoneNumber) const
{
    addressBook()->load();
    KABC::AddressBook::ConstIterator it; // is something like Addressee
    for ( it = addressBook()->begin();
            it != addressBook()->end();
            ++it )
    {
        KABC::PhoneNumber::List nrsOfPerson = it->phoneNumbers();

        int sumOfNrs = nrsOfPerson.size();

        for ( int i = 0; i < sumOfNrs; i++)
        {
            if(nrsOfPerson[i]==phoneNumber)
            {
                kDebug() << it->formattedName() << endl;
                return addressBook()->findByUid(it->uid());
            }
        }
    }
    return KABC::Addressee();
}


QPixmap KCallCore::getStatusIcon(const CallStatus & status)
{ return m_statusPix[int(status)]; }

HistoryModel * const KCallCore::historyModel() const
{ return m_historyModel; }

CallModel * const KCallCore::callModel() const
{ return m_callModel; }

void KCallCore::showDTMFInput()
{
    kDebug() << "KCallCore::showDTMFInput()" << endl;

    // DTMFInput:
    if (0 == m_dtmfInputDialog)
    {
        m_dtmfInputDialog = new KDialog();
        m_dtmfInputDialog->setCaption(i18n("DTMF tones"));
        m_dtmfInputDialog->setButtons(KDialog::Ok);
        m_dtmfInputDialog->setMainWidget(m_dtmfInput);
    }
    Q_ASSERT(0 != m_dtmfInputDialog);
    kDebug() << "KCallCore::showDTMFInput(): showing dialog." << endl;
    m_dtmfInputDialog->show();
}

#include "kcallcore.moc"
