/*
    This file is part of Kontact.

    Copyright (c) 2001 Matthias Hoelzer-Kluepfel <mhk@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

    As a special exception, permission is given to link this program
    with any edition of Qt, and distribute the resulting executable,
    without including the source code for Qt in the source distribution.
*/

#include <QWidget>
#include <QDropEvent>

#include <kaction.h>
#include <kdebug.h>
#include <kgenericfactory.h>
#include <kactioncollection.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kparts/componentfactory.h>

#include <kicon.h>
#include <kontact/core.h>
#include "kontact/plugin.h"
//#include "kontact/coreinterface.h" 
#include "kcall_plugin.h"

typedef KGenericFactory< KCallPlugin, Kontact::Core > KCallPluginFactory;
K_EXPORT_COMPONENT_FACTORY( libkontact_kcallplugin,
                            KCallPluginFactory( "kontact_kcallplugin" ) )

KCallPlugin::KCallPlugin( Kontact::Core *core, const QStringList& )
  : Kontact::Plugin( core, core, "kcall" ),
    m_interface( 0 )
{
  setComponentData( KCallPluginFactory::componentData() );

  /*
    KAction *action  = new KAction(KIcon("identity"), i18n("New Contact..."), this);
    actionCollection()->addAction("new_contact", action );
  connect(action, SIGNAL(triggered(bool)), SLOT( slotNewContact()));
  action->setShortcut(QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_C));
  insertNewAction(action);
  mUniqueAppWatcher = new Kontact::UniqueAppWatcher(
      new Kontact::UniqueAppHandlerFactory<KCallUniqueAppHandler>(), this );
  */
}

KCallPlugin::~KCallPlugin()
{
}

QString KCallPlugin::tipFile() const
{
  // TODO: tips file
  //QString file = KStandardDirs::locate("data", "kcall/tips");
  QString file;
  return file;
}

  KParts::ReadOnlyPart* KCallPlugin::createPart()
{
  KParts::ReadOnlyPart * part = loadPart();
  if ( !part ) return 0;

  // Create the stub that allows us to talk to the part
//  m_interface = new OrgKdeKAddressbookCoreInterface( "org.kde.kcall", "/KCall", QDBusConnection::sessionBus() );
  return part;
}

QStringList KCallPlugin::configModules() const
{
  QStringList modules;
  //modules << "PIM/kabconfig.desktop" << "PIM/kabldapconfig.desktop";
  return modules;
}

QStringList KCallPlugin::invisibleToolbarActions() const
{
  //return QStringList( "file_new_contact" );
}

OrgKdeKAddressbookCoreInterface *KCallPlugin::interface()
{
  if ( !m_interface ) {
    part();
  }
  Q_ASSERT( m_interface );
  return m_interface;
}

void KCallPlugin::slotNewContact()
{
//  interface()->newContact();
}

bool KCallPlugin::createDBUSInterface( const QString& serviceType )
{
  if ( serviceType == "DBUS/AddressBook" )  {
    Q_ASSERT( m_interface );
    return true;
  }

  return false;
}

void KCallPlugin::configUpdated()
{
}

bool KCallPlugin::isRunningStandalone()
{
  return mUniqueAppWatcher->isRunningStandalone();
}

bool KCallPlugin::canDecodeMimeData( const QMimeData * mimeData )
{
  return false; //mimeData->hasText() || KPIM::MailList::canDecode( mimeData );
}

void KCallPlugin::processDropEvent( QDropEvent *event )
{
    /*
  const QMimeData *md = event->mimeData();
  if ( KPIM::MailList::canDecode( md ) ) {
    event->accept();
    KPIM::MailList mails = KPIM::MailList::fromMimeData( md );
    if ( mails.count() != 1 ) {
      KMessageBox::sorry( core(),
                          i18n( "Drops of multiple mails are not supported." ) );
    } else {
      KPIM::MailSummary mail = mails.first();
      org::kde::kmail::kmail kmail("org.kde.kmail", "/KMail", QDBusConnection::sessionBus());
      QString sFrom = kmail.getFrom( mail.serialNumber() );
      if ( !sFrom.isEmpty() ) {
        KAddrBookExternal::addEmail( sFrom, core() );
      }
    }
    return;
  }
*/
  KMessageBox::sorry( core(), i18n( "Cannot handle drop events of type '%1'." ,
                        event->format() ) );
}

////

#include "../kcall_options.h"

void KCallUniqueAppHandler::loadCommandLineOptions()
{
    KCmdLineArgs::addCmdLineOptions( kcall_options() );
}

int KCallUniqueAppHandler::newInstance()
{
    kDebug(5602) << endl;
    // Ensure part is loaded
    (void)plugin()->part();

/*    org::kde::KCall::Core kcall("org.kde.kcall", "/KCall", QDBusConnection::sessionBus());
    QDBusReply<bool> reply = kcall.handleCommandLine();
    if(reply.isValid() ) {
        bool handled = reply;
        kDebug(5602) << "handled=" << handled << endl;
        if ( !handled ) // no args -> simply bring kcall plugin to front
            return Kontact::UniqueAppHandler::newInstance();    
    }
    */

    return 0;
}

#include "kcall_plugin.moc"

// vim: sw=2 sts=2 tw=80 et
