/***************************************************************************
                          import.h  -  a dialog to import notes
                             -------------------
    begin                : Tue 25 Oct 2005
    copyright            : (C) 2005 by Michal Rudolf
    email                : mrudolf@kdewebdev.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __IMPORTDIALOG_H__
#define __IMPORTDIALOG_H__

#include "ui_importbase.h"
#include <KDialog>
#include <qstringlist.h>

class ImportDialog : public KDialog, private Ui::ImportBase
{
  Q_OBJECT
public:
  enum OptionsEOL {UseNone, UseLines, UseParagraphs};
  enum OptionsEncoding {UseLocal = 0, UseUTF8 = 8};
  enum OptionsTitle {UsePath = 0, UseFilename = 16, UseFirstWord = 32, UseFirstLine = 64};
  ImportDialog(QWidget* parent = 0);
  ~ImportDialog();
  /** Number describing all of import options */
  int importOptions() const;
  /** Set all options */
  void setImportOptions(int i);
  /** Number of succesfully imported files */
  int noteCount() const;
  /** Rich text of @p i-th imported note */
  QString noteText(int i) const;
  /** Title of @p i-th imported file */
  QString noteTitle(int i) const;

public slots:
  /** Execute dialog */
  int exec();
  /** Select files to import */
  void selectFiles();
private:
  /** Import note from file, set @p title according to given flags */
  QString importNote(const QString& fileName, QString& title, int flags) const;
  /** List of all files tried */
  QStringList m_files;
  /** Names of titles for notes that where succesfully imported */
  QStringList m_titles;
  /** Rich text notes that where succesfully imported */
  QStringList m_notes;
};

#endif

