/***************************************************************************
                          knowit.cpp  -  main window
                             -------------------
    begin                : czw wrz 26 08:27:40 CEST 2002
    copyright            : (C) 2002-2004 by Micha� Rudolf
    email                : mrudolf@kdewebdev.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtimer.h>
#include <qclipboard.h>
#include <qdatetime.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <q3paintdevicemetrics.h>
#include <qwhatsthis.h>
#include <q3simplerichtext.h>
#include <qsplitter.h>

#include <kaction.h>
#include <kcmdlineargs.h>
#include <kcolordialog.h>
#include <kconfig.h>
#include <kconfiggroup.h>
#include <kdeversion.h>
#include <kedittoolbar.h>
#include <kfiledialog.h>
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
    #include <kkeydialog.h>
#endif

#include <KDebug>
#include <KFind>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kmimetype.h>
#include <kmenu.h>
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   #include <kprinter.h>
#endif

#include <KActionCollection>
#include <kstatusbar.h>
#include <ktip.h>
#include <kinputdialog.h>
#include <kglobal.h>
#include <KRecentFilesAction>
#include <KStandardAction>
#include <KToggleAction>

#include "knowit.h"
#include "knowitapp.h"
#include "knowittray.h"
#include "knowittree.h"
#include "knowitlinks.h"
#include <KXMLGUIClient>
#include <KXMLGUIFactory>
#include "importdialog.h"

const QString Knowit::Untitled = i18n("untitled") + ".kno";

namespace {
    static QAction* createAction( const QString& label,
                                  const QString& icon,
                                  const char* shortcut,
                                  QObject* recv,
                                  const char* slot,
                                  KActionCollection* parent,
                                  const char* name ) {
        QAction* action = parent->addAction( name );
        action->setText( label );
        if ( !icon.isEmpty() )
            action->setIcon( KIcon( icon ) );
        action->setShortcut( QKeySequence( shortcut ) );
        QObject::connect( action, SIGNAL(triggered(bool)), recv, slot );
        return action;
    }
    static KToggleAction* createToggleAction( const QString& label,
                                        const QString& icon,
                                        const char* shortcut,
                                        QObject* recv,
                                        const char* slot,
                                        KActionCollection* parent,
                                        const char* name ) {
        KToggleAction* action = parent->add<KToggleAction>( name );
        action->setText( label );
        if ( !icon.isEmpty() )
            action->setIcon( KIcon( icon ) );
        action->setShortcut( QKeySequence( shortcut ) );
        QObject::connect( action, SIGNAL(toggled(bool)), recv, slot );
        return action;
    }

}
Knowit::Knowit(QWidget* parent, const char *name) : KXmlGuiWindow(parent),
   FindDlg(new KFindDialog(this)),
   PrefDlg(new KnowitPreferences(this)),
   ChooserDlg(new KnowitChooser(this)),
   LinkDlg(new KnowitLinkDialog(this)),
   Notes()
{
   setObjectName(name);
   statusBar()->insertItem(i18n("Ready"), 0);
   Layout = new QSplitter(this);
   Items = new KnowitTree(Layout, "Items");
   QSplitter* EditLayout = new QSplitter(Layout);
   EditLayout->setOrientation(Qt::Vertical);
   Edit = new KnowitEdit(EditLayout, "Edit");
   Links = new KnowitLinks(EditLayout);

   connect(&FindDlg, SIGNAL(search()), this, SLOT(slotEditFind()));
   lastSought = 0;

   /* main view */
   setAutoSaveSettings();
   setCentralWidget(Layout);

   Layout->setWhatsThis(i18n("Move it to change tree/edit layout."));
   connect(Items, SIGNAL(contextMenu(K3ListView*, Q3ListViewItem*, const QPoint&)),
     this, SLOT(slotContextMenu(K3ListView*, Q3ListViewItem*, const QPoint&)));
   connect(Items, SIGNAL(selectionChanged(Q3ListViewItem*)), this,
     SLOT(slotItemChanged(Q3ListViewItem*)));
   connect(Items, SIGNAL(selectionChanged(Q3ListViewItem*)), this,
     SLOT(slotItemChanged(Q3ListViewItem*)));

   /* Edit (Q3TextEdit) */
   Edit->setMinimumSize(200, 150);
   Edit->setTextFormat(Qt::RichText);
   Edit->setWhatsThis(i18n("<h2>Note text</h2>Add text for selected note here.\n"
      "Text can be formatted, for example <b>bold</b> or <i>italic</i>."));
   connect(Edit, SIGNAL(selectionChanged()), this, SLOT(slotEditChanged()));
   connect(Edit, SIGNAL(textChanged()), this, SLOT(slotEditChanged()));
   connect(Edit, SIGNAL(cursorPositionChanged(int, int)), this,
     SLOT(slotEditCursorChanged(int, int)));
   connect(Edit, SIGNAL(textDropped(const QString&, int)),
      this, SLOT(slotLinkDropped(const QString&, int)));

   /* Links (K3ListBox) */
   connect(Links, SIGNAL(doubleClicked(Q3ListBoxItem*)),
      this, SLOT(slotShowAttachment(Q3ListBoxItem*)));
   connect(Links, SIGNAL(contextMenuRequested(Q3ListBoxItem*, const QPoint&)),
      this, SLOT(slotContextLinksMenu(Q3ListBoxItem*, const QPoint&)));
   connect(Links, SIGNAL(textDropped(const QString&, int)),
      this, SLOT(slotLinkDropped(const QString&, int)));
   connect(Links, SIGNAL(linkRemove()), this, SLOT(slotLinkRemove()));
   connect(Links, SIGNAL(linkOpen()), this, SLOT(slotLinkOpen()));
   connect(Links, SIGNAL(linkOpenWith()), this, SLOT(slotLinkOpenWith()));

   /* Actions: File */
   KStandardAction::openNew(this, SLOT(slotFileNew()), actionCollection());
   KStandardAction::open(this, SLOT(slotFileOpen()), actionCollection());
   actionRecent = KStandardAction::openRecent(this, SLOT(slotFileRecent(const KUrl&)),
      actionCollection());

   KStandardAction::save(this, SLOT(slotFileSave()), actionCollection());
   KStandardAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
   createAction(i18n("&Export to HTML..."), "document-export", "", this,
      SLOT(slotFileExport()), actionCollection(), "file_export_html");
   KStandardAction::print(this, SLOT(slotFilePrint()), actionCollection());
   createAction(i18n("Document &Information..."), "", "", this,
      SLOT(slotFileInfo()), actionCollection(), "file_info");
   KStandardAction::quit(this, SLOT(slotFileQuit()), actionCollection());

   /* Actions: Edit */
   actionCut = KStandardAction::cut(Edit, SLOT(cut()), actionCollection());
   actionCopy = KStandardAction::copy(Edit, SLOT(copy()), actionCollection());
   KStandardAction::paste(Edit, SLOT(paste()), actionCollection());
   actionUndo = KStandardAction::undo(Edit, SLOT(undo()), actionCollection());
   actionRedo = KStandardAction::redo(Edit, SLOT(redo()), actionCollection());
   actionBold = createToggleAction(i18n("Toggle &Bold"), "format-text-bold",
      "Ctrl+B", this, SLOT(slotEditBold(bool)), actionCollection(), "edit_bold");
   actionItalic = createToggleAction(i18n("Toggle &Italic"), "format-text-italic",
      "Ctrl+I", this, SLOT(slotEditItalic(bool)), actionCollection(),
      "edit_italic");
   actionUnderline = createToggleAction(i18n("Toggle &Underline"), "format-text-underline",
      "Ctrl+U", this, SLOT(slotEditUnderline(bool)), actionCollection(),
      "edit_underline");
   createAction(i18n("&Text Color..."), "format-text-color", "Ctrl+#", this,
      SLOT(slotEditColor()), actionCollection(), "edit_color");
   createAction(i18n("&Superscript"), "format-text-superscript", "", this,
      SLOT(slotEditSuperscript()), actionCollection(), "edit_superscript");
   createAction(i18n("&Subscript"), "format-text-subscript", "", this,
      SLOT(slotEditSubscript()), actionCollection(), "edit_subscript");
   createAction(i18n("&Normal Text"), "", "", this,
      SLOT(slotEditNormal()), actionCollection(), "edit_align_normal");
   createAction(i18n("&Bullet List"), "", "", this,
      SLOT(slotEditListBullet()), actionCollection(), "list_bullet");
   createAction(i18n("&Numbered list"), "", "", this,
      SLOT(slotEditListNumber()), actionCollection(), "list_number");
   createAction(i18n("&Uppercase list"), "", "", this,
      SLOT(slotEditListUpper()), actionCollection(), "list_upper");
   createAction(i18n("&Lowercase List"), "", "", this,
      SLOT(slotEditListLower()), actionCollection(), "list_lower");
   createAction(i18n("N&o List"), "", "", this,
      SLOT(slotEditListNone()), actionCollection(), "list_none");
   KStandardAction::find(&FindDlg, SLOT(exec()), actionCollection());
   KStandardAction::findNext(this, SLOT(slotEditFindNext()), actionCollection());
   createAction(i18n("&Go to &Note..."), "goto", "Ctrl+G", this,
      SLOT(slotEditGoTo()), actionCollection(), "goto_note");
   actionEditAlignLeft = createToggleAction(i18n("Align &Left"), "format-justify-left", "Ctrl+L", this,
      SLOT(slotEditAlignLeft()), actionCollection(), "edit_align_left");
   actionEditAlignRight = createToggleAction(i18n("Align &Right"), "format-justify-right", "Ctrl+R",
      this, SLOT(slotEditAlignRight()), actionCollection(), "edit_align_right");
   actionEditAlignJustify = createToggleAction(i18n("&Justify"), "format-justify-fill", "Ctrl+J",
      this, SLOT(slotEditAlignJustify()), actionCollection(), "edit_align_justify");
   actionEditAlignCenter = createToggleAction(i18n("&Center"), "format-justify-center", "Ctrl+E", this,
      SLOT(slotEditAlignCenter()), actionCollection(), "edit_align_center");
   createAction(i18n("Insert &Date"), "date", "F5", this,
      SLOT(slotEditInsertDate()), actionCollection(), "edit_insert_date");
   createAction(i18n("Insert &File..."), "", "", this,
      SLOT(slotEditInsertFile()), actionCollection(), "edit_insert_file");
   actionRawTextMode = createToggleAction(i18n("&Raw Text Mode"),
       "source", "F10", this, SLOT(slotRawTextMode(bool)),
       actionCollection(), "edit_raw_text");

   /* Actions: edit setup */
   actionCut->setEnabled(false);
   actionCopy->setEnabled(false);
   connect(Edit, SIGNAL(copyAvailable(bool)), actionCut, SLOT(setEnabled(bool)));
   connect(Edit, SIGNAL(copyAvailable(bool)), actionCopy, SLOT(setEnabled(bool)));
   connect(Edit, SIGNAL(undoAvailable(bool)), actionUndo, SLOT(setEnabled(bool)));
   connect(Edit, SIGNAL(redoAvailable(bool)), actionRedo, SLOT(setEnabled(bool)));

   /* Actions: notes */
   createAction(i18n("&Add"), "", "Alt+Insert", this, SLOT(slotNoteAdd()),
      actionCollection(), "note_add");
   createAction(i18n("Add &Subnote"), "", "Alt+Shift+Insert", this,
      SLOT(slotNoteAddChild()), actionCollection(), "note_add_child");
   createAction(i18n("&Delete"), "edit-delete", "Alt+Delete", this,
      SLOT(slotNoteRemove()), actionCollection(), "note_delete");
   createAction(i18n("&Rename"), "edit-rename", "F2", this,
      SLOT(slotNoteRename()), actionCollection(), "note_rename");
   createAction(i18n("E&xpand all"), "", "", Items,
      SLOT(slotExpandAll()), actionCollection(), "note_expand_all");
   createAction(i18n("&Expand Current"), "", "Alt++", Items,
      SLOT(slotExpand()), actionCollection(), "note_expand");
   createAction(i18n("Co&llapse All"), "", "", Items,
      SLOT(slotCollapseAll()), actionCollection(), "note_collapse_all");
   createAction(i18n("&Collapse Current"), "", "Alt+-", Items,
      SLOT(slotCollapse()), actionCollection(), "note_collapse");
   createAction(i18n("Move Up"), "go-up", "Alt+Shift+Up", this,
      SLOT(slotNoteMoveUp()), actionCollection(), "note_move_up");
   createAction(i18n("Move Down"), "go-down", "Alt+Shift+Down", this,
      SLOT(slotNoteMoveDown()), actionCollection(), "note_move_down");
   createAction(i18n("Move Level Up"), "go-previous", "Alt+Shift+Left", this,
      SLOT(slotNoteMoveLeft()), actionCollection(), "note_move_left");
   createAction(i18n("Move Level Down"), "go-next", "Alt+Shift+Right", this,
      SLOT(slotNoteMoveRight()), actionCollection(), "note_move_right");
   createAction(i18n("Move to the Top"), "go-top", "Alt+Shift+Home", this,
      SLOT(slotNoteMoveBegin()), actionCollection(), "note_move_begin");
   createAction(i18n("Move to the Bottom"), "go-bottom", "Alt+Shift+End", this,
      SLOT(slotNoteMoveEnd()), actionCollection(), "note_move_end");
   createAction(i18n("Sort"), "", "", this,
      SLOT(slotNoteSort()), actionCollection(), "note_sort");
   createAction(i18n("Insert Files..."), "", "", this,
      SLOT(slotNoteFromFile()), actionCollection(), "note_from_file");
   createAction(i18n("Statistics..."), "", "", this,
      SLOT(slotNoteStatistics()), actionCollection(), "note_statistics");

   /* Actions: Links */
   createAction(i18n("&Add Link..."), "mail-attachment", "Ctrl+Shift+I", this,
      SLOT(slotLinkAdd()), actionCollection(), "link_add");
   createAction(i18n("&Remove Link"), "edit-delete", "", this,
      SLOT(slotLinkRemove()), actionCollection(), "link_remove");
   createAction(i18n("Open Link"), "document-open", "", this,
      SLOT(slotLinkOpen()), actionCollection(), "link_open");
   createAction(i18n("Open Link With..."), "document-open", "", this,
      SLOT(slotLinkOpenWith()), actionCollection(), "link_open_with");
   createAction(i18n("&Modify Link..."), "", "", this,
      SLOT(slotLinkModify()), actionCollection(), "link_modify");
   createAction(i18n("&Copy Link Location"), "edit-copy", "", this,
      SLOT(slotLinkCopy()), actionCollection(), "link_copy");

   /* Actions: Options */
   KStandardAction::keyBindings(this, SLOT(slotOptionsKeys()), actionCollection());
   KStandardAction::configureToolbars(this, SLOT(slotOptionsToolbar()), actionCollection());
   KStandardAction::preferences(this, SLOT(slotOptions()), actionCollection());

   /* Actions: Help */
   createAction(i18n("&Tip of the Day"), "idea", "", this,
      SLOT(slotHelpTip()), actionCollection(), "help_tip");

   /* Actions: nonplugged */
   createAction(i18n("Minimize"), "", "", this, SLOT(showMinimized()),
      actionCollection(), "window_minimize");
   createAction(i18n("Switch Area"), "", "Ctrl+M", this,
      SLOT(slotEditSwitch()), actionCollection(), "edit_switch");
   createAction(i18n("Overwrite Mode"), "", "Insert", this,
      SLOT(slotOverwriteModeChange()), actionCollection(), "overwrite_mode_change");
   createAction(i18n("Go to Previous Note"), "", "Alt+Up", Items,
      SLOT(slotItemUp()), actionCollection(), "note_previous");
   createAction(i18n("Go to Next Note"), "", "Alt+Down", Items,
      SLOT(slotItemDown()), actionCollection(), "note_next");
   createAction(i18n("Go to First Note"), "go-first", "Alt+Home", Items,
      SLOT(slotItemBegin()), actionCollection(), "note_begin");
   createAction(i18n("Go to Last Note"), "go-last", "Alt+End", Items,
      SLOT(slotItemEnd()), actionCollection(), "note_end");
   createAction(i18n("Go to First Subnote"), "", "Alt+Right", Items,
      SLOT(slotItemRight()), actionCollection(), "note_right");
   createAction(i18n("Go to Parent Note"), "go-up", "Alt+Left", Items,
      SLOT(slotItemLeft()), actionCollection(), "note_left");

   /* Actions */
   setStandardToolBarMenuEnabled(true);
   createGUI("knowitui.rc");


   /* Insert/overwrite mode */
   statusBar()->insertPermanentFixedItem(("WWW"), StatusOvr);
   statusBar()->changeItem(i18nc("editor insert mode in statusbar", "INS"), StatusOvr);
   connect(statusBar(), SIGNAL(pressed(int)), this,
      SLOT(slotOverwriteModeChange(int)));

   /* System tray */
   systray = new KnowitTray(this, "Systray");
   systray->setIcon(Notes.Pixmaps[1]);
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   systray->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
#else
   kWarning() << "Code temporarily removed in Knowit::Knowit";
#endif


   /* Autosave */
   AutosaveTimer = new QTimer(this);
   AutosaveTimer->setSingleShot( true );
   connect(AutosaveTimer, SIGNAL(timeout()), this, SLOT(slotFileAutosave()));

   /* Configuration */
   config = KGlobal::config();
   readOptions();

   if (runMinimized)
      showMinimized();
   else
       show();

   bool opened = false;
   KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
   for(int i = 0; i < args->count(); i++)
       if (open(args->url(i))) {
          opened = true; break;
          }
   args->clear();

   if (!opened && Options.reopen && filename.path() != "")
      open(KUrl(filename));

   slotActionUpdate();

   KTipDialog::showTip(this);

   shuttingDown = false;

   applyMainWindowSettings(KConfigGroup(config, "MainWindow"));

   /* Dialogs */
   m_importDlg = new ImportDialog(this);


   /* check if knowituirc was found */
   QWidget *w = factory()->container("notes_popup", this);
   if (!w)
      KMessageBox::error(this, i18n("<qt><p>Cannot open resource file <b>knowitui.rc</b>. "
          "Some of the menus and toolbars will be missing.</p>"
          "<p>Probably Knowit was not installed properly. "
    "If you installed from source, please invoke <i>make install</i>."
    "If you use precompiled package, please contact packager.</p></qt>"));
}

Knowit::~Knowit()
{
   delete systray;
   delete AutosaveTimer;
   delete m_importDlg;
}

bool Knowit::open(const KUrl& fname)
{
   QFile file(fname.path());
   if (!file.open(QIODevice::ReadOnly)) {
      KMessageBox::error(this, i18n("<qt>Cannot open file<br/><b>%1</b></qt>", fname.url()));
      if (filename == fname)
         filename = "";
      return false;
      }
   uint prevdepth = 0;
   reset();
   QTextStream ts(&file);
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   ts.setEncoding(QTextStream::UnicodeUTF8);
#else
   kWarning() << "Code temporarily removed in Knowit::open";
#endif

   QString s = "";
   Q3ListViewItem* current = 0;
   Q3ListViewItem* active = 0;
   Notes.autoUpdate = false;
   while (!s.isNull()) {
      s = ts.readLine();
      if (s.left(9) == "\\NewEntry" || s.left(13) == "\\CurrentEntry") {
         QString name;
         uint depth;
         QTextStream ls(&s);
         ls >> name >> depth;
         name = ls.readLine();
         name = name.trimmed();
         if (depth == prevdepth) /* same level */
            if (depth) current = new K3ListViewItem(current->parent(), current, name);
            else current = new K3ListViewItem(Items, current, name);
         else if (depth > prevdepth)   /* subnote */
            current = new K3ListViewItem(current, name);
         else {   /* parent note */
            for(uint i=depth; i<prevdepth && current; i++)
               current = current->parent();
            if (current && current->parent())
               current = new K3ListViewItem(current->parent(), current, name);
            else current = new K3ListViewItem(Items, current, name);
         }
         prevdepth = depth;
         Notes.addNote(current);
         Notes.find(current)->open(ts);
         if (s.left(13) == "\\CurrentEntry")
            active = current;
      }
   }
   Notes.autoUpdate = true;
   Notes.updateNotesView();
   filename = fname;
   setCaption(fname.fileName());
   Notes.modified = false;
   Edit->clear();
   slotNoteChanged(active);
   Edit->setModified(false);
   actionRecent->addUrl(filename);
   slotStatusMsg(i18n("File %1 opened.", filename.fileName()));
   slotActionUpdate();

   file.close();
   if (Options.backup)
     URLOperator.copy(fname.path(), fname.path() + "~", false, false);
   if (Options.autosave)
     AutosaveTimer->start(Options.autosave * 60 * 1000);
   return true;
}

bool Knowit::save(const KUrl& fname)
{
  QFile file(fname.path());
  if (!file.open(QIODevice::WriteOnly)) {
     KMessageBox::error(this, i18n("<qt>Cannot save file<br><b>%1</b></qt>", fname.url()));
     return false;
     }
  if (Items->prevItem && Edit->isModified())
     Notes.changeNote(Items->prevItem, Edit->length() ? Edit->text() : QString::null);

  QTextStream ts(&file);
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   ts.setEncoding(QTextStream::UnicodeUTF8);
#else
   kWarning() << "Code temporarily removed in Knowit::save";
#endif

  for (Q3ListViewItemIterator it(Items); it.current(); it++)
     Notes.find(it.current())->save(ts, it.current() == Items->currentItem());
  filename = fname;
  setCaption(fname.fileName());
  Notes.modified = false;
  Edit->setModified(false);
  actionRecent->addUrl(filename);
  slotStatusMsg(i18n("File %1 saved.", filename.fileName()));
  if (Options.autosave)
     AutosaveTimer->start(Options.autosave * 60 * 1000);
  return true;
}

void Knowit::reset()
{
   Items->prevItem = 0;
   Notes.clearNotes();
   Items->clear();
   Edit->clear();
   filename = "";
   setCaption(Untitled);
   Notes.modified = false;
   slotActionUpdate();
}



void Knowit::closeEvent(QCloseEvent* e)
{
   if (!Options.docked || ((KnowitApplication*)kapp)->closeByLogout()) {
      saveOptions();
      KMainWindow::closeEvent(e);
      }
   else
      hide();
}

bool Knowit::queryClose()
{
   if (!modified()) return shuttingDown = true;
   else if (Options.unconditionalSave) {
       slotFileSave();
       return shuttingDown = filename.isEmpty() || !modified()  || (KMessageBox::questionYesNo(this,
           i18n("<qt>File <b>%1</b> cannot be saved. Quit anyway?</qt>", filename.fileName()),
           i18n("Cannot Save"), KGuiItem(i18n("Quit Without Saving")), KStandardGuiItem::cancel())
           == KMessageBox::Yes);
       }
   else
      switch (KMessageBox::questionYesNoCancel(this,
       i18n("<qt>File <b>%1</b> was modified. Do you want to save it?</qt>",
          filename.isEmpty() ? Untitled : filename.fileName()), i18n("Save On Exit"),
          KStandardGuiItem::save(), KStandardGuiItem::discard(), KStandardGuiItem::cancel())) {
      case KMessageBox::Yes:
         slotFileSave();
         return shuttingDown = !modified();
      case KMessageBox::No:
         return shuttingDown = true;
      default:
         return false;
      }
}

bool Knowit::queryExit()
{
   if (!shuttingDown) {
      /* brutally closed by session manager */
      if (modified())
         save(filename.isEmpty() ? KUrl(i18n("untitled.kno")) : filename);
      saveOptions();
   }
   return true;
}


void Knowit::find(Q3ListViewItem* start)
{
   const bool cs = FindDlg.options() & KFind::CaseSensitive;
   if (soughtText.isEmpty() || !Items->firstChild()) return;
   if (start && lastSought == start &&
      Edit->find(soughtText, cs, false))
      return;
   if (!start) start = Items->firstChild();
   Q3ListViewItemIterator it(start);
   while (it.current()) {
      QString s = Notes.text(it.current());
      if (lastSought != it.current() &&
         s.contains(soughtText, cs ? Qt::CaseSensitive : Qt::CaseInsensitive)) {
         slotNoteChanged(it.current());
         Edit->find(soughtText, cs, false);
         lastSought = it.current();
         return;
      }
      it++;
   }
   KMessageBox::information(this, i18n("<qt>Sought text:<br/><b>%1</b><br/>not found.</qt>", soughtText));
}


void Knowit::saveOptions()
{
  KConfigGroup group(config, "MainWindow");
  group.writeEntry("Widgets", Layout->sizes());
  Options.save(config);
  group = KConfigGroup(config, "General");
  group.writeEntry("Minimized", isMinimized() || !isVisible());
  group.writeEntry("LastFile", filename.path());
  group.writeEntry("Geometry", size());
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
  actionCollection()->writeShortcutSettings("Keys", group);
#else
   kWarning() << "Code temporarily removed in Knowit::saveOptions";
#endif
  config->deleteGroup("Recent Files");
  actionRecent->saveEntries(KConfigGroup(config,"Recent Files"));
  saveMainWindowSettings(group);
}

void Knowit::readOptions()
{
   KConfigGroup group(config, "MainWindow");
   QList<int> sizes = group.readEntry("Widgets", QList<int>());
   if (sizes.count() == 2)
      Layout->setSizes(sizes);
   Options.read(config);
   group = KConfigGroup(config, "General");
   runMinimized = group.readEntry("Minimized", false);
   QString oldfile = group.readEntry("LastFile");
   QSize defSize = size();
   resize(group.readEntry("Geometry", defSize));
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   actionCollection()->readShortcutSettings("Keys", group);
#else
   kWarning() << "Code temporarily removed in Knowit::readOptions";
#endif
   actionRecent->loadEntries(KConfigGroup(config,"Recent Files"));
   if (!oldfile.isEmpty())
      filename = KUrl(oldfile);
   applyOptions(Options);
}

void Knowit::slotNoteChanged(Q3ListViewItem* item)
{
   if (item) {
      if (Options.autoCollapse)
         Items->slotCollapseAll();
      Items->setCurrentItem(item);
      Items->setSelected(item, true);
      Items->ensureItemVisible(item);
   }
}

bool Knowit::modified()
{
   return Notes.modified || (Items->currentItem() && Edit->isModified());
}

void Knowit::applyOptions(const KnowitOptions& O, bool store)
{
   if (store) Options = O;
   if (O.docked)
      systray->show();
   else
      systray->hide();

   if (O.customColors) {
      QPalette p = Edit->palette();
      p.setColor( QPalette::Foreground, QColor(Options.foreEditColor));
      p.setColor( QPalette::Background, QColor(Options.backEditColor));
      Edit->setPalette(p);
   }
   else Edit->setPalette(QPalette());

   if (Options.customFont)
      Edit->setFont(QFont(Options.fontFamily, Options.fontSize));
   else
      Edit->setFont(QFont());

   Edit->setWordWrap(O.wordwrap ? Q3TextEdit::WidgetWidth : Q3TextEdit::NoWrap);
   Edit->setTabChangesFocus(O.tabfocus);
   if (!O.autosave)
       AutosaveTimer->stop();
   else
       AutosaveTimer->start(O.autosave * 60 * 1000);
   Edit->swapEOL = O.enterBreakLine;
   Layout->setOrientation(O.horizontalSplit ? Qt::Horizontal :
         Qt::Vertical);
   Items->setAlternateBackground(O.alternateTree ? QColor(O.alternateColor) : QColor());
   Items->repaintContents();
   if (Items->currentItem())
      slotItemChanged(Items->currentItem());
   slotNoteChanged(Items->currentItem());
}

TNote* Knowit::currentNote() const
{
   if (!Items->currentItem()) return 0;
   return Notes.find(Items->currentItem());
}






/*** Slots ***/
void Knowit::slotStatusMsg(const QString &text)
{
   statusBar()->clearMessage();
   statusBar()->changeItem(text, 0);
}


void Knowit::slotContextMenu(K3ListView*, Q3ListViewItem*, const QPoint& p)
{
   QMenu *w = qobject_cast<QMenu*>(factory()->container("notes_popup", this));
   if (w)
      w->exec(p);
}


void Knowit::slotContextLinksMenu(Q3ListBoxItem*, const QPoint& p)
{
   QMenu *w = qobject_cast<QMenu*>(factory()->container("links_popup", this));
   if (w)
      w->exec(p);
}



/******* Slots: Notes *******/
void Knowit::slotNoteAdd()
{
   bool Ok;
   K3ListViewItem* elt;
   QString text = KInputDialog::getText(i18n("Add note"),
      i18n("Note title:"), Options.defaultName, &Ok);
   if (!Ok) return;
   if (Items->currentItem())
      if (Items->currentItem()->parent())
         elt = new K3ListViewItem(Items->currentItem()->parent(),
            Items->currentItem(), text);
      else
         elt = new K3ListViewItem(Items, Items->currentItem(), text);
   else
      elt = new K3ListViewItem(Items, text);
   Notes.addNote(elt);
   slotNoteChanged(elt);
   if (Notes.count() == 1)
      slotActionUpdate();
}

void Knowit::slotNoteAddChild()
{
   if (!Items->currentItem()) {
      KMessageBox::error(this, i18n("You have to choose a note first before adding a subnote."));
      return;
   }
   bool Ok;
   QString text = KInputDialog::getText(i18n("Add Subnote"),
      i18n("Note title:"), Options.defaultName, &Ok);
   if (!Ok) return;
   Q3ListViewItem* lastChild = Items->currentItem()->firstChild();
   while (lastChild && lastChild->nextSibling())
      lastChild = lastChild->nextSibling();

   K3ListViewItem* elt;
   if (lastChild)
       elt = new K3ListViewItem(Items->currentItem(), lastChild, text);
   else
      elt = new K3ListViewItem(Items->currentItem(), text);
   Notes.addNote(elt);
   slotNoteChanged(elt);
}

void Knowit::slotNoteRemove()
{
   Q3ListViewItem* elt = Items->currentItem();
   if (!elt) return;
   QString msg = elt->childCount() ?
      i18n("<qt>Are you sure you want to delete note<br/><b>%1</b><br/> "
         "and its subnotes?</qt>", elt->text(0)) :
      i18n("<qt>Are you sure you want to delete note<br/><b>%1</b>?</qt>", elt->text(0));
   if (KMessageBox::questionYesNo(this,
                                  msg,
                                  i18n("Delete Note"),
                                  KStandardGuiItem::del(),
                                  KStandardGuiItem::cancel()) == KMessageBox::Yes) {
      Q3ListViewItem* parent = elt->parent();
      Notes.removeNote(elt);
      if (!Notes.count())
         slotActionUpdate();
      if (!parent)
         parent = Items->firstChild();
      Items->prevItem = 0;
      slotNoteChanged(parent);
      }
}

void Knowit::slotNoteRename()
{
    if (Items->currentItem()) {
       Items->currentItem()->setRenameEnabled(0, true);
       Items->currentItem()->startRename(0);
       }
}

void Knowit::slotNoteStatistics()
{
  int depth = 0;
  int words = TNotesCollection::wordCount(Edit->text());
  int chars = TNotesCollection::charCount(Edit->text());
  int pars = TNotesCollection::parsCount(Edit->text());
  int allwords = 0;
  int allpars = 0;
  int allchars = 0;
  for (Q3ListViewItemIterator it(Items); it.current(); ++it)
  {
    if (!(*it)->childCount() && !(*it)->nextSibling() && (*it)->depth() > depth)
      depth = (*it)->depth();
  }
  Q_FOREACH( TNote* const it, Notes.notesList() )
  {
    allwords += TNotesCollection::wordCount((*it).text);
    allpars += TNotesCollection::parsCount((*it).text);
    allchars += TNotesCollection::charCount((*it).text);
  }
  KMessageBox::information(this, i18n("<qt><br><b>Current note:</b><br>"
  "Characters: %1<br>Words: %2<br>Paragraphs: %3<br><br>"
  "<b>Current file:</b><br>Number of notes: %4<br>Maximum depth: %5<br>"
  "Characters: %6<br>Words: %7<br>Paragraphs: %8</qt>",
    chars, words, pars, Notes.count(), depth, allchars, allwords, allpars),
    i18n("Statistics"));
}




void Knowit::slotItemChanged(Q3ListViewItem* i)
{
   if (Items->prevItem && Edit->isModified())
      Notes.changeNote(Items->prevItem, Edit->length() ? Edit->text() : QString());
   Items->prevItem = i;
   QString s;
   if (!Items->prevItem || (s = Notes.text(Items->prevItem)).isEmpty())
      Edit->setText(QString::null);
   else
      Edit->setText(s);
   Edit->setModified(false);
   Links->clear();
   TNote* note = Notes.find(i);
   if (note)
      for (int j=0; j<note->linkCount(); j++)
         Links->insertItem(note->link(j).icon(), note->link(j).text(Options.linkFormat));
   if (Links->count())
      Links->show();
   else
      Links->hide();
}

void Knowit::slotNoteMoveUp()
{
   Q3ListViewItem* elt = Items->currentItem();
   if (elt && elt->itemAbove()) {
      Q3ListViewItem* above = elt->parent() ? elt->parent()->firstChild()
         : Items->firstChild();
      if (above == elt)
         return;
      if (!above->nextSibling() || above->nextSibling() == elt)
         above = 0;
      else while (above->nextSibling()->nextSibling() != elt)
         above = above->nextSibling();
      if (above)
         elt->moveItem(above);
      else {
         Q3ListViewItem* parent = elt->parent();
         Items->takeNode(elt);
         Items->insertNode(parent, elt);
      }
      slotNoteChanged(elt);
      Notes.modified = true;
   }
}

void Knowit::slotNoteMoveDown()
{
   Q3ListViewItem* elt = Items->currentItem();
   if (elt && elt->nextSibling()) {
      elt->moveItem(elt->nextSibling());
      slotNoteChanged(elt);
      Notes.modified = true;
      }
}

void Knowit::slotNoteMoveBegin()
{
   Q3ListViewItem* elt = Items->currentItem();
   if (elt) {
      Q3ListViewItem* parent = elt->parent();
      Items->takeNode(elt);
      Items->insertItem(elt);
      if (parent)
         Notes.find(parent)->updateView();
      slotNoteChanged(elt);
      Notes.modified = true;
      }
}

void Knowit::slotNoteMoveEnd()
{
   Q3ListViewItem* elt = Items->currentItem();
   if (!elt)
      return;
   Q3ListViewItem* last = Items->firstChild();
   Q3ListViewItem* parent = elt->parent();
   while (last->nextSibling())
      last = last->nextSibling();
   if (elt && elt != last) {
      Items->takeNode(elt);
      Items->insertNode(0, elt, last);
      slotNoteChanged(elt);
      if (parent)
         Notes.find(parent)->updateView();
      Notes.modified = true;
      }
}

void Knowit::slotNoteMoveLeft()
{
   Q3ListViewItem* elt = Items->currentItem();
   Q3ListViewItem* parent = elt->parent();
   if (elt && parent) {
      Items->takeNode(elt);
      Items->insertNode(parent->parent(), elt, parent);
      Notes.find(parent)->updateView();
      slotNoteChanged(elt);
      Notes.modified = true;
      }
}

void Knowit::slotNoteMoveRight()
{
   Q3ListViewItem* elt = Items->currentItem();
   Q3ListViewItem* above = elt->itemAbove();
   if (elt && above) {
      if (above->depth() < elt->depth())
         return;
      else if (above->depth() == elt->depth()) {
         Items->takeNode(elt);
	 Items->insertNode(above, elt);
	 }
      else {
         while (above->depth() > elt->depth()+1)
            above = above->parent();
         Items->takeNode(elt);
         Items->insertNode(above->parent(), elt, above);
	 }
      Notes.find(above)->updateView();
      slotNoteChanged(elt);
      Notes.modified = true;
   }
}

void Knowit::slotNoteSort()
{
   if (Items->currentItem()) {
      Items->currentItem()->sortChildItems(0, true);
      Items->repaintContents();
   }
}

void Knowit::slotNoteFromFile()
{
  m_importDlg->setImportOptions(Options.importFlags);
  if (m_importDlg->exec() == QDialog::Accepted)
  {
    K3ListViewItem* elt;
    Q3ListViewItem* parent = Items->currentItem();
    K3ListViewItem* after = 0;
    for (int i = 0; i<m_importDlg->noteCount(); i++)
    {
      if (parent)
        elt = new K3ListViewItem(parent, after, m_importDlg->noteTitle(i));
      else
        elt = new K3ListViewItem(Items, after, m_importDlg->noteTitle(i));
      after = elt;
      Notes.addNote(elt, m_importDlg->noteText(i));
    }
    Options.importFlags = m_importDlg->importOptions();
  }
}




void Knowit::slotFileNew()
{
   if (queryClose()) reset();
}

void Knowit::slotFileOpen()
{
   if (!queryClose()) return;
   KUrl url=KFileDialog::getOpenUrl(KUrl(),
      i18n("*.kno|KnowIt files (*.kno)\n*|All files"), this, i18n("Open File..."));
   if (!url.isEmpty())
      open(url);
}

void Knowit::slotFileSave()
{
  if (filename.isEmpty())
     slotFileSaveAs();
  else {
     save(filename);
     }
}

void Knowit::slotFileAutosave()
{
  if (modified())
     slotFileSave();
  if (modified()) {	/* file wasn't saved */
     KMessageBox::sorry(this, i18n("<qt><p>File could not be saved automatically."
          "<p>Autosaving disabled - save file manually to turn autosaving on.</p></qt>"));
     AutosaveTimer->stop();
     }
}


void Knowit::slotFileRecent(const KUrl& url)
{
   if (queryClose())
      if (!open(url))
         actionRecent->removeUrl(url);
}


void Knowit::slotFileSaveAs()
{
   KUrl url=KFileDialog::getSaveUrl(QDir::currentPath(),
        i18n("*.kno|KnowIt files (*.kno)\n*|All files"), this, i18n("Save as..."));
   if (!url.isEmpty()) {
      if (!url.fileName().contains('.'))
        url = KUrl(url.path() + ".kno");
      QFileInfo fileinfo(url.path());
      if (fileinfo.exists() && KMessageBox::questionYesNo(this,
         i18n("<qt>File<br><b>%1</b><br>already exists. Overwrite it?</qt>", url.path()),
         i18n("Overwrite File?"), KStandardGuiItem::overwrite(), KStandardGuiItem::cancel()) == KMessageBox::No)
         return;
      save(url);
  }
}


void Knowit::slotFileInfo()
{
   int count = 0;
   Q3ListViewItemIterator it(Items);
   while (it.current()) {
      count++;
     it++;
   }
   KMessageBox::information(this, i18n("<qt><h1>%1</h1>"
      "Document path: %2<br>"
      "Number of notes: %3</qt>", filename.fileName(), filename.path(), QString::number(count)));
}


void Knowit::slotFilePrint()
{
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   KPrinter printer;
   printer.setFullPage(true);
   if (Edit->text().trimmed().isEmpty() || !printer.setup(this))
      return;

   Q3SimpleRichText richText(Edit->text(), Edit->font());
   QPainter p(&printer);
   Q3PaintDeviceMetrics metrics(p.device());
   int dpix = metrics.logicalDpiX();
   int dpiy = metrics.logicalDpiY();
   const int margin = 72; // pt
   QRect body(margin*dpix / 72, margin*dpiy / 72,
      metrics.width()-  margin*dpix /72*2,
      metrics.height()-margin*dpiy/72*2);
   richText.setWidth(&p, body.width());
   QRect view(body);
   int page = 1;
   while (true) {
      richText.draw(&p, body.left(), body.top(), view, colorGroup());
         view.moveBy(0, body.height());
      p.translate( 0 , -body.height());
      p.setFont(Edit->font());
      p.drawText( view.right() - p.fontMetrics().width( QString::number(page)),
         view.bottom() + p.fontMetrics().ascent() + 5, QString::number(page));
      if (view.top() >= body.top() + richText.height())
         break;
      printer.newPage();
      page++;
   }
#else
   kWarning() << "Code temporarily removed in Knowit::slotFilePrint";
#endif

}




void Knowit::slotFileQuit()
{
   if (queryClose()) {
      saveOptions();
      kapp->quit();
      }
}

void Knowit::slotFileExport()
{
  if (!Items->firstChild()) return;         /* nothing to export */
  ChooserDlg->setChoice(Options.exportFlags);
  if (ChooserDlg->exec() != QDialog::Accepted) return; /* cancelled */

  /* Deduce filename from Knowit document name */
  QString htmlfile;
  if (filename.isValid()) {
     htmlfile = filename.path();
     if (htmlfile.right(4) == ".kno")
        htmlfile = htmlfile.left(htmlfile.length() - 4) + ".html";
     else htmlfile = "";
  }
  if (htmlfile.isEmpty())
     htmlfile == QDir::currentPath();

  /* Confirm/choose filename */
  KUrl url=KFileDialog::getSaveUrl(htmlfile,
        i18n("*.html|HTML files (*.html)\n*|All files"), this, i18n("Export to HTML"));
  if (!url.isEmpty()) {
     if (!url.fileName().contains('.'))
        url = KUrl(url.path() + ".html");
     QFileInfo fileinfo(url.path());
     if (fileinfo.exists() && KMessageBox::questionYesNo(this,
         i18n("<qt>File<br><b>%1</b><br>already exists. Overwrite it?</qt>", url.path()),
         i18n("Overwrite File?"),
         KStandardGuiItem::overwrite(),
         KStandardGuiItem::cancel() ) == KMessageBox::No)
        return;
     int choice = ChooserDlg->getChoice();
     Q3ListViewItem* start = (choice & KnowitChooser::SaveAll) ?
         Items->firstChild() : Items->currentItem();
     QString origname = filename.isEmpty() ? Untitled : filename.path();
     QString style;
     if (choice & KnowitChooser::Style)
        style = QString("body {font-family: \"%1\"; color: %2; background-color: %3}")
           .arg(Edit->family(),
                Edit->palette().color(QPalette::Foreground).name(),
                Edit->palette().color(QPalette::Background).name());
     Notes.find(start)->saveHTML(url, origname, style, choice);
     Options.exportFlags = choice;
  }
}

void Knowit::slotEditBold(bool bold)
{
   Edit->setBold(bold);
}

void Knowit::slotEditItalic(bool italic)
{
   Edit->setItalic(italic);
}

void Knowit::slotEditUnderline(bool underline)
{
   Edit->setUnderline(underline);
}

void Knowit::slotEditColor()
{
   QColor color = Edit->color();
   if (KColorDialog::getColor(color) == QDialog::Accepted)
      Edit->setColor(color);
}

void Knowit::slotEditSuperscript()
{
   Edit->setVerticalAlignment(Q3TextEdit::AlignSuperScript);
}

void Knowit::slotEditSubscript()
{
   Edit->setVerticalAlignment(Q3TextEdit::AlignSubScript);
}

void Knowit::slotEditNormal()
{
   Edit->setVerticalAlignment(Q3TextEdit::AlignNormal);
}

void Knowit::slotEditAlignLeft()
{
   Edit->setAlignment(Qt::AlignLeft);
}

void Knowit::slotEditAlignRight()
{
   Edit->setAlignment(Qt::AlignRight);
}

void Knowit::slotEditAlignJustify()
{
   Edit->setAlignment(Qt::AlignJustify);
}

void Knowit::slotEditAlignCenter()
{
   Edit->setAlignment(Qt::AlignCenter);
}

void Knowit::slotEditListBullet()
{
   Edit->setParagType(Q3StyleSheetItem::DisplayListItem, Q3StyleSheetItem::ListDisc);
}

void Knowit::slotEditListNumber()
{
   Edit->setParagType(Q3StyleSheetItem::DisplayListItem, Q3StyleSheetItem::ListDecimal);
}

void Knowit::slotEditListUpper()
{
   Edit->setParagType(Q3StyleSheetItem::DisplayListItem, Q3StyleSheetItem::ListUpperAlpha);
}

void Knowit::slotEditListLower()
{
   Edit->setParagType(Q3StyleSheetItem::DisplayListItem, Q3StyleSheetItem::ListLowerAlpha);
}

void Knowit::slotEditListNone()
{
   Edit->setParagType(Q3StyleSheetItem::DisplayBlock, Q3StyleSheetItem::ListDisc);
}

void Knowit::slotEditFind()
{
   if (soughtText != FindDlg.pattern()) {
      soughtText = FindDlg.pattern();
      lastSought = 0;
      find();
      }
   else
      find(Items->currentItem());
}

void Knowit::slotEditFindNext()
{
   if (soughtText.isEmpty()) FindDlg.exec();
   find(Items->currentItem());
}

void Knowit::slotEditGoTo()
{
   bool Ok;
   QString page = KInputDialog::getText(i18n("Go to"),
      i18n("Go to page with given title"), "", &Ok);
   if (Ok && !page.isEmpty()) {
      Q3ListViewItem* item = Items->findItem(page, 0);
      if (!item) item = Items->findItem(page, 0, Q3ListView::ExactMatch);
      if (!item) item = Items->findItem(page, 0, Q3ListView::Contains);
      slotNoteChanged(item);
   }
}

void Knowit::slotEditInsertDate()
{
   if (Items->currentItem()) {
      bool ib = actionBold->isChecked(), ii=actionItalic->isChecked(), iu=actionUnderline->isChecked();
      QColor color = Edit->color();
      Edit->setItalic(Options.insertDateItalic);
      Edit->setBold(Options.insertDateBold);
      Edit->setUnderline(Options.insertDateUnderline);
      Edit->setColor(QColor(Options.insertDateColor));
      Edit->insert(QDateTime::currentDateTime().toString(Options.insertDateFormat));
      Edit->setItalic(ii);
      Edit->setBold(ib);
      Edit->setUnderline(iu);
      Edit->setColor(color);

   }
}

void Knowit::slotEditInsertFile()
{
   if (!Items->currentItem())
      return;
   KUrl url=KFileDialog::getOpenUrl(KUrl(),
      i18n("*|All files"), this, i18n("Insert file..."));
   if (url.isEmpty())
      return;
   QFile file(url.path());
   if (!file.open(QIODevice::ReadOnly)) {
      KMessageBox::error(this, i18n("<qt>Cannot open file<br><b>%1</b></qt>", url.url()));
      return;
      }
   QTextStream ts(&file);
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   ts.setEncoding(QTextStream::UnicodeUTF8);
#else
   kWarning() << "Code temporarily removed in slotEditInsertFile";
#endif

   QString s, text;
   while (true) {
      s = ts.readLine();
      if (s.isNull())
         break;
      if (s.isEmpty())
         text += '\n';
      else
         text += s;
      }
   Edit->insert(text);
}


void Knowit::slotEditSwitch()
{
   if (Items->hasFocus())
      Edit->setFocus();
   else Items->setFocus();
}

void Knowit::slotEditChanged()
{
   actionBold->setChecked(Edit->bold());
   actionItalic->setChecked(Edit->italic());
   actionUnderline->setChecked(Edit->underline());
   actionEditAlignLeft->setChecked(Edit->alignment() == Qt::AlignLeft);
   actionEditAlignRight->setChecked(Edit->alignment() == Qt::AlignRight);
   actionEditAlignCenter->setChecked(Edit->alignment() == Qt::AlignCenter);
   actionEditAlignJustify->setChecked(Edit->alignment() == Qt::AlignJustify);
}

void Knowit::slotEditCursorChanged(int, int)
{
   slotEditChanged();
}



void Knowit::slotOptions()
{
   PrefDlg->setOptions(Options);
   if (PrefDlg->exec() == QDialog::Accepted)
      applyOptions(PrefDlg->getOptions());
}

void Knowit::slotOptionsKeys()
{
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   KKeyDialog K(false);
   K.configure(actionCollection());
#else
   kWarning() << "Code temporarily removed in Knowit::slotOptionsKeys";
#endif
}

void Knowit::slotOptionsToolbar()
{
   KEditToolBar dlg(actionCollection());
   if (dlg.exec())
      createGUI();
}


void Knowit::slotUndock()
{
   Options.docked = false;
   applyOptions(Options);
}

void Knowit::slotHelpTip()
{
   KTipDialog::showTip(this, 0, true);
}


void Knowit::slotActionUpdate()
{
   if (Items->childCount())
      stateChanged("notesPresent");
   else
      stateChanged("notesPresent", KXMLGUIClient::StateReverse);
   Edit->setEnabled(Notes.count());
}

void Knowit::slotOverwriteModeChange(int i)
{
   if (i == StatusOvr)
      slotOverwriteModeChange();
}

void Knowit::slotOverwriteModeChange()
{
   Edit->setOverwriteMode(!Edit->isOverwriteMode());
   statusBar()->changeItem(Edit->isOverwriteMode() ? i18n("OVR") :
      i18n("INS"), StatusOvr);
}

void Knowit::slotRawTextMode(bool raw)
{
   if (!Items->currentItem())
      return;
   if (Items->prevItem && Edit->isModified())
      Notes.changeNote(Items->prevItem, Edit->length() ? Edit->text() : QString());
   Edit->setTextFormat(raw ? Qt::PlainText : Qt::RichText);
   Edit->setText(Notes.text(Items->currentItem()));
}

void Knowit::slotShowAttachment(Q3ListBoxItem*)
{
   slotLinkOpen();
}

void Knowit::slotLinkAdd()
{
   slotLinkDropped("", 0);
}

void Knowit::slotLinkDropped(const QString& s, int flags)
{
   TNoteLink link;
   link.link = s;
   if (!flags && !LinkDlg->modifyLink(link))
      return;
   currentNote()->addLink(link);
   Links->insertItem(link.icon(), link.text(Options.linkFormat));
   if (Links->count() == 1)
      Links->show();
   if (Options.firstLink) {
      KMessageBox::information(this, i18n("You have just added your first link. "
      "Please remember that only links are stored in Knowit document, "
      "not linked files. If you move your document to other computer, "
      "links to local files probably won't work."));
      Options.firstLink = false;
   }
}

void Knowit::slotLinkRemove()
{
   if (Links->currentItem() != -1 && KMessageBox::questionYesNo(this,
      i18n("<qt>Are you sure you want to remove link:<br><b>%1</b>?</qt>", Links->currentText()),
      i18nc("dialog caption", "Remove Link"), KGuiItem(i18nc("button label", "Remove Link")),
      KStandardGuiItem::cancel()) == KMessageBox::Yes) {
      currentNote()->removeLink(Links->currentItem());
      Links->removeItem(Links->currentItem());
      if (!Links->count())
         Links->hide();
      }
}

void Knowit::slotLinkOpen()
{
   if (Links->currentItem() == -1)
       return;
   TNoteLink link = currentNote()->link(Links->currentItem());
   if (link.isLocalReference()) {
      QString s = link.link.remove(0, 9);
      Q3ListViewItem* item = Items->findItem(s, 0);
      if (!item) Items->findItem(s, 0, Q3ListView::BeginsWith);
      if (!item) Items->findItem(s, 0, Q3ListView::Contains);
      slotNoteChanged(item);
   }
   else
      link.open();
}

void Knowit::slotLinkOpenWith()
{
   if (Links->currentItem() != -1)
       currentNote()->link(Links->currentItem()).openWith();
}

void Knowit::slotLinkModify()
{
   if (Links->currentItem() == -1) return;
   TNoteLink link = currentNote()->link(Links->currentItem());
   if (LinkDlg->modifyLink(link)) {
      currentNote()->modifyLink(Links->currentItem(), link);
      Links->changeItem(link.icon(), link.text(Options.linkFormat),
         Links->currentItem());
      }
}

void Knowit::slotLinkCopy()
{
   if (Links->currentItem() == -1) return;
   TNoteLink link = currentNote()->link(Links->currentItem());
   kapp->clipboard()->setText(link.link, QClipboard::Clipboard);
   kapp->clipboard()->setText(link.link, QClipboard::Selection);
}

