/***************************************************************************
                          knowitpref.cpp  -  description
                             -------------------
    begin                : czw lis 28 2002
    copyright            : (C) 2002-2004 by Micha� Rudolf
    email                : mrudolf@kdewebdev.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "knowitpref.h"


#include <qcheckbox.h>
#include <qcolor.h>
#include <qtoolbutton.h>
#include <qspinbox.h>
#include <qdatetime.h>
#include <qlayout.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qlabel.h>
#include <q3vbox.h>

#include <kconfig.h>
#include <KConfigGroup>
#include <kcombobox.h>
#include <KDebug>
#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kdeversion.h>
#include <kfontdialog.h>
#include <klineedit.h>
#include <kcolorbutton.h>
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   #include <kdualcolorbutton.h>
#endif


void KnowitOptions::save(KSharedConfigPtr config) const
{
   KConfigGroup group(config, "KDE");
   group.writeEntry("MultipleInstances", multipleInstances);
   group = KConfigGroup(config,"General");
   group.writeEntry("Docked", docked);
   group.writeEntry("Autosave", autosave);
   group.writeEntry("Reopen", reopen);
   group.writeEntry("HorizontalSplit", horizontalSplit);
   group.writeEntry("UnconditionalSave", unconditionalSave);
   group.writeEntry("Backup", backup);
   group.writeEntry("DefaultName", defaultName);
   group.writeEntry("AlternateTree", alternateTree);
   group.writeEntry("AlternateColor", alternateColor);
   group.writeEntry("AutoCollapse", autoCollapse);

   group = KConfigGroup(config,"Links");
   group.writeEntry("First", firstLink);
   group.writeEntry("Format", linkFormat);

   group = KConfigGroup(config,"Editor");
   group.writeEntry("WordWrap", wordwrap);
   group.writeEntry("EnterBreakLine", enterBreakLine);
   group.writeEntry("Qt::TabFocus", tabfocus);
   group.writeEntry("CustomColors", customColors);
   group.writeEntry("ForeEditColor", foreEditColor);
   group.writeEntry("BackEditColor", backEditColor);
   group.writeEntry("CustomFont", customFont);
   group.writeEntry("FontFamily", fontFamily);
   group.writeEntry("FontSize", fontSize);


   group = KConfigGroup(config,"Templates");
   group.writeEntry("InsertDateFormatString", insertDateFormat);
   group.writeEntry("InsertDateColor", insertDateColor);
   group.writeEntry("InsertDateBold", insertDateBold);
   group.writeEntry("InsertDateItalic", insertDateItalic);
   group.writeEntry("InsertDateUnderline", insertDateUnderline);


   group = KConfigGroup(config,"Export");
   group.writeEntry("Flags", exportFlags);


   group = KConfigGroup(config,"Import");
   group.writeEntry("Flags", importFlags);
}


void KnowitOptions::read(KSharedConfigPtr config)
{
   KConfigGroup group(config, "KDE");
   multipleInstances = group.readEntry("MultipleInstances", true);
   group = KConfigGroup(config,"General");
   docked = group.readEntry("Docked", false);
   autosave = group.readEntry("Autosave", 0);
   reopen = group.readEntry("Reopen", true);
   horizontalSplit = group.readEntry("HorizontalSplit", true);
   defaultName = group.readEntry("DefaultName", i18n("New note"));
   alternateTree = group.readEntry("AlternateTree", false);
   alternateColor = group.readEntry("AlternateColor", QString("LightYellow"));
   autoCollapse = group.readEntry("AutoCollapse", false);
   unconditionalSave = group.readEntry("UnconditionalSave", false);
   backup = group.readEntry("Backup", true);

   group = KConfigGroup(config,"Links");
   firstLink = group.readEntry("First", true);
   linkFormat = group.readEntry("Format", 0);

   group = KConfigGroup(config,"Editor");
   wordwrap = group.readEntry("WordWrap", true);
   enterBreakLine = group.readEntry("EnterBreakLine", true);
   tabfocus = group.readEntry("Qt::TabFocus", false);
   customColors = group.readEntry("CustomColors", false);
   foreEditColor = group.readEntry("ForeEditColor", QString("Black"));
   backEditColor = group.readEntry("BackEditColor", QString("White"));
   customFont = group.readEntry("CustomFont", false);
   fontFamily = group.readEntry("FontFamily", QString("Helvetica"));
   fontSize = group.readEntry("FontSize", 10); // KDE-Default?

   group = KConfigGroup(config,"Templates");
   insertDateFormat = group.readEntry("InsertDateFormatString", QString("ddd d MMM yyyy, hh:mm:ss"));
   insertDateColor = group.readEntry("InsertDateColor", QString("Blue"));
   insertDateBold = group.readEntry("InsertDateBold", false);
   insertDateItalic = group.readEntry("InsertDateItalic", true);
   insertDateUnderline = group.readEntry("InsertDateUnderline", false);


   group = KConfigGroup(config,"Export");
   exportFlags = group.readEntry("Flags", 0);


   group = KConfigGroup(config,"Import");
   importFlags = group.readEntry("Flags", 0);
}



KnowitPreferences::KnowitPreferences(QWidget* parent, Qt::WindowFlags flags)
   : KPageDialog(parent, flags)
{
    setFaceType( List );
    setWindowTitle(i18n("KnowIt Preferences"));
    setButtons(Ok|Cancel);
    setDefaultButton(Ok);
    QWidget* page;
    KPageWidgetItem* pageItem;
    QVBoxLayout* layout;

    /* first page: General */
    pageItem = addPage( page= new QWidget, i18n("General"));
    pageItem->setHeader(i18n("General options"));
    pageItem->setIcon(KIcon("configure"));
    layout = new QVBoxLayout(page);
    layout->setMargin(0);
    layout->setSpacing(spacingHint());
    layout->addWidget(docked = new QCheckBox(i18n("&Dock in System tray"), page));
    docked->setObjectName("Dock");
    layout->addWidget(reopen = new QCheckBox(i18n("Open &last file on startup"), page));
    reopen->setObjectName("Reopen");

    Q3HBox* autosaveBox = new Q3HBox(page);
    layout->addWidget(autosaveBox);
    new QLabel(i18n("Autosave:"), autosaveBox);
    autosave = new QSpinBox(autosaveBox);
    autosave->setRange(0,120);
    autosave->setSingleStep(5);
    autosave->setObjectName("Autosave");
    autosave->setPrefix(i18n("every "));
    autosave->setSuffix(i18n(" min."));
    autosave->setSpecialValueText(i18n("Never"));
    autosave->setWhatsThis(i18n("Current file will be automatically saved "
       "after the specified interval. Set to <i>never</i> to disable autosave."));

    layout->addWidget(unconditionalSave = new QCheckBox(i18n("Automatically save file on e&xit"),
        page));
    unconditionalSave->setObjectName("UnconditionalSave");
    unconditionalSave->setWhatsThis(i18n("Current file will be automatically saved "
       "on exit without confirmation."));
    layout->addWidget(backup = new QCheckBox(i18n("Create &backups"),
        page));
    backup->setObjectName("Backup");
    backup->setWhatsThis(i18n("Create backup of current document before it is saved."));
    layout->addWidget(multipleInstances = new QCheckBox(i18n("Allow &multiple instances of KnowIt"),
        page));
    multipleInstances->setObjectName("Instances");
    multipleInstances->setWhatsThis(i18n("If this option is disabled, only one "
       "instance of KnowIt will be allowed. If there is another instance already running, "
       "it will be automatically activated instead of running new one."));
    layout->addStretch(1);


    /* second page: Interface */
    pageItem = addPage( page = new QWidget, i18n("Interface"));
    pageItem->setHeader(i18n("Interface options"));
    pageItem->setIcon(KIcon("misc"));
    layout = new QVBoxLayout(page);
    layout->setMargin(0);
    layout->setSpacing(spacingHint());
    layout->addWidget(horizontalSplit = new QCheckBox(i18n("Split window &horizontally"), page));
    horizontalSplit->setObjectName("Split");
    horizontalSplit->setWhatsThis(i18n("If this option is set, notes tree will "
       "be displayed on the left and the editor on the right (this is default).<br>"
       "Otherwise notes tree will be displayed at the top and the editor at the bottom."));

    Q3HBox* defaultNameBox = new Q3HBox(page);
    layout->addWidget(defaultNameBox);
    new QLabel(i18n("Default note name:"), defaultNameBox);
    defaultName = new KLineEdit(defaultNameBox);
    defaultName->setObjectName("DefaultName");
    defaultName->setWhatsThis(i18n("Default name for new notes. It would be "
       "autoselected, so if you often copy text using mouse selection, you may "
       "consider setting default name to none."));

    Q3HBox* linkBox = new Q3HBox(page);
    layout->addWidget(linkBox);
    new QLabel(i18n("Link format:"), linkBox);
    linkFormat = new KComboBox(linkBox);
    linkFormat->setObjectName("LinkFormat");
    linkFormat->addItem(i18n("Description (link)"));
    linkFormat->addItem(i18n("Link (description)"));
    linkFormat->addItem(i18n("Link only"));
    linkFormat->addItem(i18n("Description only"));

    Q3HBox* alternateBox = new Q3HBox(page);
    layout->addWidget(alternateBox);
    alternateTree = new QCheckBox(i18n("Alternate colors in tree"), alternateBox);
    alternateTree->setObjectName("Alternate");
    alternateColor = new KColorButton(Qt::white, alternateBox);
    alternateColor->setObjectName("AlternateColor");
    layout->addWidget(autoCollapse = new QCheckBox(i18n("Automatically collapse other notes"), page));
    autoCollapse->setObjectName("AutoCollapse");
    autoCollapse->setWhatsThis(i18n("If this option is set, only current subtree "
       "will be visible, other notes will be automatically collapsed."));
    layout->addStretch(1);


    /* third page: Editor */
    pageItem = addPage( page = new QWidget, i18n("Editor"));
    pageItem->setHeader(i18n("Editor options"));
    pageItem->setIcon(KIcon("edit"));
    layout = new QVBoxLayout(page);
    layout->setMargin(0);
    layout->setSpacing(spacingHint());
    layout->addWidget(wordwrap = new QCheckBox(i18n("Use &word wrap"),
       page));
    wordwrap->setObjectName("WordWrap");
    layout->addWidget(enterBreakLine = new QCheckBox(i18n("'Enter' ends current line, not paragraph"),
       page));
    enterBreakLine->setObjectName("EnterLineBreak");
    layout->addWidget(tabfocus = new QCheckBox(i18n("'Tab' in editor changes focus"),
       page));
    tabfocus->setObjectName("Qt::TabFocus");
    Q3HBox* colorBox = new Q3HBox(page);
    layout->addWidget(colorBox);
    customColors = new QCheckBox(i18n("Use &custom colors"), colorBox);
    customColors->setObjectName("CustomColors");
    #ifdef KDE4PORTING_TEMPORARILY_REMOVED
       editColors = new KDualColorButton(colorBox);
    #else
       kWarning() << "Code temporarily removed in KnowitPreferences";
    #endif


    const char fontSizes[] = {4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,22,24,26,28,32,48,64,0};
    QStringList fontList;
    KFontChooser::getFontList(fontList, false);

    Q3HBox* fontBox = new Q3HBox(page);
    layout->addWidget(fontBox);
    customFont = new QCheckBox(i18n("Use custom font:"), fontBox);
    customFont->setObjectName("customFont");
    fontFamily = new KComboBox(true, fontBox);
    fontFamily->addItems(fontList);
    fontSize = new KComboBox(true, fontBox);
    for( int i=0; fontSizes[i] != 0; i++ ){
       fontSize->addItem(QString::number(fontSizes[i]));
    }

    layout->addStretch(1);


    /* fourth page: Templates */
    pageItem = addPage(page = new QWidget, i18n("Templates"));
    pageItem->setHeader(i18n("Templates configuration"));
    pageItem->setIcon(KIcon("wizard"));
    layout = new QVBoxLayout(page);
    layout->setMargin(0);
    layout->setSpacing(spacingHint());
    page->setWhatsThis(i18n("<html>\n"
"<p>These expressions may be used: </p>\n"
"<table>\n"
"<tr><th>d</th><td>the day as number without a leading zero (1-31)</td></tr>\n"
"<tr><th>dd</th><td>the day as number with a leading zero (01-31)</td></tr>\n"
"<tr><th>ddd</th><td>the abbreviated localized day name (e.g. 'Mon'..'Sun')</td></tr>\n"
"<tr><th>dddd</th><td>the long localized day name (e.g. 'Monday'..'Sunday')</td></tr>\n"
"<tr><th>M</th><td>the month as number without a leading zero (1-12)</td></tr>\n"
"<tr><th>MM</th><td>the month as number with a leading zero (01-12)</td></tr>\n"
"<tr><th>MMM</th><td>the abbreviated localized month name (e.g. 'Jan'..'Dec')</td></tr>\n"
"<tr><th>MMMM</th><td>the long localized month name (e.g. 'January'..'December')</td></tr>\n"
"<tr><th>yy</th><td>the year as two digit number (00-99)</td></tr>\n"
"<tr><th>yyyy</th><td>the year as four digit number (1752-8000)</td></tr>\n"
"<tr><th>h</th><td>the hour without a leading zero (0..23 or 1..12 if AM/PM display)</td></tr>\n"
"<tr><th>hh</th><td>the hour with a leading zero (00..23 or 01..12 if AM/PM display)</td></tr>\n"
"<tr><th>m</th><td>the minute without a leading zero (0..59)</td></tr>\n"
"<tr><th>mm</th><td>the minute with a leading zero (00..59)</td></tr>\n"
"<tr><th>s</th><td>the second without a leading zero (0..59)</td></tr>\n"
"<tr><th>ss</th><td>the second with a leading zero (00..59)</td></tr>\n"
"<tr><th>AP</th><td>use AM/PM display</td></tr>\n"
"<tr><th>AP</th><td>use am/pm display</td></tr>\n"
"</table></html>"));

    Q3HBox* topBox = new Q3HBox(page);
    layout->addWidget(topBox);
    QLabel * label1 = new QLabel(i18n("Date Format: "), topBox);
    label1->setFixedSize(label1->sizeHint());
    insertDatePreview = new QLabel("", topBox);

    Q3HBox *tmpBox = new Q3HBox(page);
    layout->addWidget(tmpBox);
    insertDateColorButton = new KColorButton(Qt::blue, tmpBox);
    insertDateColorButton->setObjectName("InsertDateColorButton");
    insertDateFormatEdit = new KLineEdit(tmpBox);
    insertDateFormatEdit->setObjectName("InsertDateFormatEdit");
    insertDateItalicButton = new QToolButton(tmpBox);
    insertDateItalicButton->setObjectName("InsertDateItalicButton");
    insertDateItalicButton->setIcon(KIcon("text_italic"));
    insertDateItalicButton->setCheckable(true);
    insertDateBoldButton = new QToolButton(tmpBox);
    insertDateBoldButton->setObjectName("InsertDateBoldButton");
    insertDateBoldButton->setIcon(KIcon("text_bold"));
    insertDateBoldButton->setCheckable(true);
    insertDateUnderlineButton = new QToolButton(tmpBox);
    insertDateUnderlineButton->setObjectName("InsertDateUnderlineButton");
    insertDateUnderlineButton->setIcon(KIcon("text_under"));
    insertDateUnderlineButton->setCheckable(true);
    insertDateColorButton->setFixedSize(insertDateBoldButton->sizeHint());
    insertDateFormatEdit->setMinimumWidth(6*insertDateBoldButton->sizeHint().width());
    layout->addStretch(1);
}

void KnowitPreferences::setOptions(const KnowitOptions& Options)
{
   docked->setChecked(Options.docked);
   reopen->setChecked(Options.reopen);
   autosave->setValue(Options.autosave);
   unconditionalSave->setChecked(Options.unconditionalSave);
   backup->setChecked(Options.backup);
   multipleInstances->setChecked(Options.multipleInstances);

   horizontalSplit->setChecked(Options.horizontalSplit);
   defaultName->setText(Options.defaultName);
   alternateTree->setChecked(Options.alternateTree);
   alternateColor->setColor(QColor(Options.alternateColor));
   autoCollapse->setChecked(Options.autoCollapse);
   linkFormat->setCurrentIndex(Options.linkFormat);

   wordwrap->setChecked(Options.wordwrap);
   enterBreakLine->setChecked(Options.enterBreakLine);
   tabfocus->setChecked(Options.tabfocus);
   customColors->setChecked(Options.customColors);
#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   editColors->setForeground(QColor(Options.foreEditColor));
   editColors->setBackground(QColor(Options.backEditColor));
#else
   kWarning() << "Code temporarily removed in KnowitPreferences::setOptions";
#endif

   customFont->setChecked(Options.customFont);
   fontFamily->setCurrentIndex(fontFamily->findText(Options.fontFamily));
   fontSize->setCurrentIndex(fontFamily->findText(QString::number(Options.fontSize)));

   insertDateFormatEdit->setText(Options.insertDateFormat);
   insertDateColorButton->setColor(QColor(Options.insertDateColor));
   insertDateBoldButton->setChecked(Options.insertDateBold);
   insertDateItalicButton->setChecked(Options.insertDateItalic);
   insertDateUnderlineButton->setChecked(Options.insertDateUnderline);
   slotUpdateDatePreview();
   connect(insertDateColorButton, SIGNAL(clicked()), this, SLOT(slotUpdateDatePreview()));
   connect(insertDateFormatEdit, SIGNAL(textChanged (const QString &)), this, SLOT(slotUpdateDatePreview()));
   connect(insertDateItalicButton, SIGNAL(toggled(bool)), this, SLOT(slotUpdateDatePreview()));
   connect(insertDateBoldButton, SIGNAL(toggled(bool)), this, SLOT(slotUpdateDatePreview()));
   connect(insertDateUnderlineButton, SIGNAL(toggled(bool)), this, SLOT(slotUpdateDatePreview()));

   /* non-GUI options */
   firstLink = Options.firstLink;
   exportFlags = Options.exportFlags;
}

KnowitOptions KnowitPreferences::getOptions() const
{
   KnowitOptions Options;
   Options.docked = docked->isChecked();
   Options.reopen = reopen->isChecked();
   Options.autosave = autosave->value();
   Options.unconditionalSave = unconditionalSave->isChecked();
   Options.multipleInstances = multipleInstances->isChecked();
   Options.backup = backup->isChecked();

   Options.horizontalSplit = horizontalSplit->isChecked();
   Options.defaultName = defaultName->text();
   Options.alternateTree = alternateTree->isChecked();
   Options.alternateColor = alternateColor->color().name();
   Options.autoCollapse = autoCollapse->isChecked();
   Options.linkFormat = linkFormat->currentIndex();

   Options.wordwrap = wordwrap->isChecked();
   Options.enterBreakLine = enterBreakLine->isChecked();
   Options.tabfocus = tabfocus->isChecked();
   Options.customColors = customColors->isChecked();

#ifdef KDE4PORTING_TEMPORARILY_REMOVED
   Options.foreEditColor = editColors->foreground().name();
   Options.backEditColor = editColors->background().name();
#else
   kWarning() << "Code temporarily removed in KnowitPreferences::getOptions";
#endif

   Options.customFont = customFont->isChecked();
   Options.fontFamily = fontFamily->currentText();
   Options.fontSize = fontSize->currentText().toInt();

   Options.insertDateFormat = insertDateFormatEdit->text();
   Options.insertDateColor = insertDateColorButton->color().name();
   Options.insertDateBold = insertDateBoldButton->isChecked();
   Options.insertDateItalic = insertDateItalicButton->isChecked();
   Options.insertDateUnderline = insertDateUnderlineButton->isChecked();

   Options.firstLink = firstLink;
   Options.exportFlags = exportFlags;
   return Options;
}

void KnowitPreferences::slotUpdateDatePreview()
{
   QString str = "<html><p align=\"center\">";
   if(insertDateBoldButton->isChecked()) str += "<b>";
   if(insertDateItalicButton->isChecked()) str +="<i>";
   if(insertDateUnderlineButton->isChecked()) str += "<u>";
   str += "<font COLOR=\""+insertDateColorButton->color().name()+"\">";
   str += QDateTime::currentDateTime().toString(insertDateFormatEdit->text());
   str += "</font>";
   if(insertDateUnderlineButton->isChecked()) str += "</u>";
   if(insertDateItalicButton->isChecked()) str +="</i>";
   if(insertDateBoldButton->isChecked()) str += "</b>";
   str += "</p></html>";
   QPalette p = insertDatePreview->palette();
   p.setColor(QPalette::Background, Qt::white);
   insertDatePreview->setPalette(p);
   insertDatePreview->setText(str);
}


