/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_ABSTRACTPROXYFEED_H
#define KRSS_ABSTRACTPROXYFEED_H

#include "krss_export.h"
#include "feed.h"
#include "item.h"

class KIcon;
class QString;

#include <boost/function.hpp>

namespace Akonadi {
    class Item;
}

namespace KRss {

class AbstractProxyFeedPrivate;

class KRSS_EXPORT AbstractProxyFeed : public Feed
{
    Q_OBJECT

public:

    explicit AbstractProxyFeed( QObject *parent = 0 );
    ~AbstractProxyFeed();

    Feed::Id id() const;

    QString title() const;
    void setTitle( const QString &title );
    QString description() const;
    void setDescription( const QString &description );
    QString type() const;
    void setType( const QString &type );
    KIcon icon() const;
    void setIcon( const KIcon &icon );
    QList<TagId> tags() const;
    void setTags( const QList<TagId> &tags );
    void addTag( const TagId &tag );
    void removeTag( const TagId &tag );

    int total() const;
    int unread() const;
    Item findItemById( Item::Id id ) const;
    bool containsItem( Item::Id id ) const;

    int fetchInterval() const;
    void setFetchInterval( int minutes );
    int maxArticleNumber() const;
    void setMaxArticleNumber( int number );
    int maxArticleAge() const;
    void setMaxArticleAge( int age );

    ItemListJob* createListJob();

    void addSourceFeed( Feed *feed );
    void removeSourceFeed( Feed *feed );
    void clearAllSourceFeeds();
    QList<Feed*> sourceFeeds() const;

public Q_SLOTS:

    void fetch() const;
    void abortFetch() const;

protected:

    virtual boost::function1<bool, const Item&> filterFunction() const = 0;

private Q_SLOTS:

    void slotFeedFetchStarted( KRss::Feed *feed );
    void slotFeedFetchPercent( KRss::Feed *feed, uint percentage );
    void slotFeedFetchFinished( KRss::Feed *feed );
    void slotFeedFetchFailed( KRss::Feed *feed, const QString &errorMessage );

    void slotFeedChanged( KRss::Feed *feed );
    void slotFeedRemoved( KRss::Feed *feed );

    void slotFeedItemsAdded( KRss::Feed *feed, const QList<KRss::Item> &items );
    void slotFeedItemsChanged( KRss::Feed *feed, const QList<KRss::Item> &items );
    void slotFeedItemsRemoved( KRss::Feed *feed, const QList<KRss::Item> &items );


private:

    Q_DISABLE_COPY(AbstractProxyFeed)
    AbstractProxyFeedPrivate * const d;
};

} // namespace KRss

#endif // KRSS_ABSTRACTPROXYFEED_H
