/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_FEEDLIST_H
#define KRSS_FEEDLIST_H

#include "krss_export.h"
#include "feed.h"

#include <boost/shared_ptr.hpp>

#include <KJob>

class QStringList;

namespace KRss {

class RetrieveFeedListJob;
class RetrieveFeedListJobPrivate;

class FeedListPrivate;

class KRSS_EXPORT FeedList : public QObject
{
    Q_OBJECT
    friend class ::KRss::RetrieveFeedListJob;
    friend class ::KRss::RetrieveFeedListJobPrivate;

public:

    explicit FeedList( const QStringList &resourceIdentifiers, QObject *parent = 0 );
    ~FeedList();

#if 0
    QList<const Feed*> feeds() const;
#endif
    QList<Feed*> feeds() const;

    QList<Feed::Id> feedIds() const;
    QList<TagId> tags() const;
    bool isEmpty() const;

    const Feed* findById( const Feed::Id& id ) const;
    Feed* findById( const Feed::Id& id );

    QList<Feed*> findByTitle( const QString &title );
    QList<const Feed*> findByTitle( const QString &title ) const;

    void fetch( Feed::Id id );

Q_SIGNALS:

    void feedAdded( KRss::Feed *feed );
    void feedChanged( KRss::Feed *feed );
    void feedRemoved( KRss::Feed *feed );
    void totalCountChanged( KRss::Feed *feed, int );
    void unreadCountChanged( KRss::Feed *feed, int );

    void fetchStarted( KRss::Feed *feed );
    void fetchPercent( KRss::Feed *feed, uint percent );
    void fetchFinished( KRss::Feed *feed );
    void fetchFailed( KRss::Feed *feed, const QString &errorMessage );

private:

    friend class FeedListPrivate;
    FeedListPrivate * const d;

    Q_DISABLE_COPY( FeedList )
    Q_PRIVATE_SLOT( d, void slotFeedAdded( KRss::Feed::Id id ) )
    Q_PRIVATE_SLOT( d, void slotFeedRemoved( KRss::Feed *feed ) )
    Q_PRIVATE_SLOT( d, void slotCollectionLoadDone( KJob *job ) )
};



class KRSS_EXPORT RetrieveFeedListJob : public KJob
{
    Q_OBJECT
    friend class ::KRss::RetrieveFeedListJobPrivate;

public:
    enum Error {
        CouldNotRetrieveFeedListError=KJob::UserDefinedError //TODO ...
    };

    explicit RetrieveFeedListJob( QObject* parent=0 );
    ~RetrieveFeedListJob();

    void setResourceIdentifiers( const QStringList& resourceIdentifiers );
    QStringList resourceIdentifiers() const;

    /* impl */ void start();

    boost::shared_ptr<FeedList> feedList() const;

private:
    RetrieveFeedListJobPrivate* const d;
    Q_PRIVATE_SLOT( d, void doStart() )
    Q_PRIVATE_SLOT( d, void loadDone(KJob*) )
};

} // namespace KRss

#endif // KRSS_FEEDLIST_H
