/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_FEEDLISTMODEL_H
#define KRSS_FEEDLISTMODEL_H

#include "krss_export.h"

#include <QtCore/QAbstractItemModel>

#include <boost/shared_ptr.hpp>

namespace KRss {

class Feed;
class FeedList;
class TagProvider;

class KRSS_EXPORT FeedListModel : public QAbstractItemModel
{
    Q_OBJECT

public:

    enum Roles {
        TierRole = Qt::UserRole + 1,
        FeedIdRole,
        FeedRole,
        TagIdRole,
        TagRole
    };

    enum Tier {
        TagTier,
        FeedTier
    };

public:

    explicit FeedListModel( const boost::shared_ptr<const FeedList>& feedlist, const TagProvider *provider,
                            QObject *parent = 0 );
    ~FeedListModel();

    boost::shared_ptr<const FeedList> feedList() const;

    QVariant headerData( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
    int columnCount( const QModelIndex &parent = QModelIndex() ) const;
    int rowCount( const QModelIndex &parent = QModelIndex() ) const;
    QVariant data( const QModelIndex &index, int role = Qt::DisplayRole ) const;
    QModelIndex index( int row, int column, const QModelIndex &parent = QModelIndex() ) const;
    QModelIndex parent( const QModelIndex &index ) const;

protected:

    // helper methods
    QList<QModelIndex> feedIndexes( Feed *feed ) const;

private:

    friend class FeedListModelPrivate;
    class FeedListModelPrivate * const d;

    Q_DISABLE_COPY( FeedListModel )
    Q_PRIVATE_SLOT( d, void slotTagCreated( const KRss::Tag& tag ) )
    Q_PRIVATE_SLOT( d, void slotTagModified( const KRss::Tag& tag ) )
    Q_PRIVATE_SLOT( d, void slotTagDeleted( const KRss::TagId& id ) )
    Q_PRIVATE_SLOT( d, void slotFeedAdded( KRss::Feed *feed ) )
    Q_PRIVATE_SLOT( d, void slotFeedChanged( KRss::Feed *feed ) )
    Q_PRIVATE_SLOT( d, void slotFeedRemoved( KRss::Feed *feed ) )
    Q_PRIVATE_SLOT( d, void slotFeedItemCountChanged( KRss::Feed *feed, int count ) )
};

} // namespace KRss

Q_DECLARE_METATYPE(KRss::Feed*)

#endif // KRSS_FEEDLISTMODEL_H
