/*
    Copyright (C) 2009    Frank Osterfeld <osterfeld@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "importitemsjob.h"
#include "libkrss/rssitem.h"
#include "rssitemsync.h"

#include <KLocalizedString>

#include <QFile>
#include <QString>

ImportItemsJob::ImportItemsJob( const Akonadi::Collection& collection, QObject* parent ) : KJob( parent ), m_collection( collection ), m_reader( 0 ) {

}

ImportItemsJob::~ImportItemsJob() {
    delete m_reader;
}

void ImportItemsJob::setSourceFile( const QString& fn ) {
    m_fileName = fn;
}

void ImportItemsJob::start() {
    QMetaObject::invokeMethod( this, "doStart", Qt::QueuedConnection );
}

void ImportItemsJob::doStart() {
    m_file.setFileName( m_fileName );
    if ( !m_file.open( QIODevice::ReadOnly ) ) {
        setError( CouldNotOpenFileFailed );
        setErrorText( i18n("Could not open file %1 for reading.", m_fileName ) );
        return;
    }
    m_reader = new ItemImportReader( &m_file );
    readBatch();
}

void ImportItemsJob::readBatch() {
    int num = 500;

    Akonadi::Item::List items;

    while ( num > 0 && m_reader->hasNext() ) {
        const KRss::RssItem rssItem = m_reader->nextItem();
        Akonadi::Item item;
        item.setRemoteId( rssItem.guid() );
        item.setMimeType( "application/rss+xml" );
        item.setPayload<KRss::RssItem>( rssItem );
        items.append( item );
        --num;
    }

    if ( items.isEmpty() ) {
        emitResult();
        return;
    }

    syncItems( items );
}

void ImportItemsJob::syncItems( const Akonadi::Item::List& items ) {
    Akonadi::ItemSync *syncer = new RssItemSync( m_collection, this );
    QObject::connect( syncer, SIGNAL( result( KJob* ) ), this, SLOT( syncDone( KJob* ) ) );
    syncer->setIncrementalSyncItems( items, Akonadi::Item::List() );
}

void ImportItemsJob::syncDone( KJob* job ) {
    if ( job->error() ) {
        setError( ItemSyncFailed );
        setErrorText( job->errorText() );
        emitResult();
        return;
    }
   readBatch();
}
