/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_IMPORTOPMLJOB_H
#define KRSS_IMPORTOPMLJOB_H

#include "libkrss/feedcollection.h"

#include <kio/job.h>
#include <KJob>

#include <QtCore/QHash>

class QByteArray;
class ImportOpmlJobPrivate;

class ImportOpmlJob : public KJob
{
    Q_OBJECT

public:

    enum Error {
        RootCollectionImportFailed = KJob::UserDefinedError,
        OpmlReadFailed,
        OpmlParsingFailed,
        FeedImportFailed
    };

public:

    explicit ImportOpmlJob( const QString &filepath, const KRss::FeedCollection &rootCollection, QObject *parent = 0 );
    ~ImportOpmlJob();

    void setDefaultTag( const QString &defaultTag );

    QHash<KRss::FeedCollection::Id, KRss::FeedCollection> importedFeeds() const;
    KRss::FeedCollection importedRootCollection() const;

    void start();
    QString errorString() const;

private Q_SLOTS:

    void doStart();
    void slotRootCollectionImportDone( KJob *job );
    void slotOpmlOpen( KIO::Job *job );
    void slotData( KIO::Job *job, const QByteArray &data );
    void slotFileResult( KJob *job );
    void slotFeedImportDone( KJob *job );

private:

    void checkDone();

    void readUnknownElement();
    void readOpml();
    void readBody();
    void readOutline( QStringList &tags );

private:

    ImportOpmlJobPrivate * const d;
};

#endif // KRSS_IMPORTOPMLJOB_H
