/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rssitemsync.h"

#include "libkrss/rssitem.h"

#include <KDebug>

RssItemSync::RssItemSync( const Akonadi::Collection &collection, QObject *parent )
        : ItemSync( collection, parent )
{
}

RssItemSync::~RssItemSync()
{
}

bool RssItemSync::updateItem( const Akonadi::Item &storedItem, Akonadi::Item &newItem )
{
        if ( storedItem.mimeType() != QString::fromLatin1( "application/rss+xml" ) ||
             newItem.mimeType() != QString::fromLatin1( "application/rss+xml" ) ) {
                kWarning() << "Either storedItem or newItem doesn't have mimetype \'application/rss+xml\'";
                kWarning() << "Remote id:" << storedItem.remoteId();
                return false;
        }

        if ( !storedItem.hasPayload() || !newItem.hasPayload() ) {
                kWarning() << "Either storedItem or newItem doesn't have payload";
                kWarning() << "Remote id:" << storedItem.remoteId();
                return false;
        }

        const KRss::RssItem newRssItem = newItem.payload<KRss::RssItem>();
        const int newHash = newRssItem.hash();
        const int storedHash = storedItem.payload<KRss::RssItem>().hash();

        if ( !newRssItem.guidIsHash() && storedHash != newHash ) {
                kDebug() << "The article has been updated:" << newItem.remoteId();
                // dont overwrite the existing flags
                // and clear \Seen + set \Recent
                newItem.setFlags( storedItem.flags() );
                newItem.clearFlag( "\\Seen" );
                newItem.setFlag( "\\Recent" );
                return true;
        }

        return false;
}
