/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_RSSRESOURCE_H
#define KRSS_RSSRESOURCE_H

#include "libkrss/feedcollection.h"
#include "libkrss/feed.h"
#include "libkrss/tag.h"

#include <akonadi/resourcebase.h>

#include <QtCore/QHash>
#include <QtDBus/QDBusMessage>

class KJob;
class FetchFeedJob;

class RssResource : public Akonadi::ResourceBase, public Akonadi::AgentBase::Observer
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface","org.kde.krss")

public:

    explicit RssResource( const QString &id );

    void collectionAdded( const Akonadi::Collection &collection, const Akonadi::Collection &parent );
    void collectionChanged( const Akonadi::Collection &collection );
    void collectionRemoved( const Akonadi::Collection &collection );
    void itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection );
    void itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &partIdentifiers );
    void itemRemoved( const Akonadi::Item &item );

public Q_SLOTS:

    void configure( WId windowId );

    // D-Bus interface
    KRss::Feed::Id addFeed( const QString& xmlUrl, const QString& subscriptionLabel );
    void fetchFeed( const KRss::Feed::Id& id );
    void abortFetch( const KRss::Feed::Id& id );
    bool removeFeed( const KRss::Feed::Id& id );
    bool subscribe( const KRss::Feed::Id& id, const QString& subscriptionLabel );
    bool unsubscribe( const KRss::Feed::Id& id, const QString& subscriptionLabel );
    QStringList subscriptions( const KRss::Feed::Id& id );
    bool addTag( const KRss::Feed::Id& id, const QString &tag );
    bool removeTag( const KRss::Feed::Id& id, const QString &tag );
    QStringList tags( const KRss::Feed::Id& id );
    bool importOpml( const QString &path, const QString &defaultTag );
    bool importItems( const QString& xmlUrl, const QString& path );
    void exportOpml( const QString &path );

Q_SIGNALS:

    void fetchStarted( const KRss::Feed::Id& id );
    void fetchPercent( const KRss::Feed::Id& id, uint percentage );
    void fetchFinished( const KRss::Feed::Id& id );
    void fetchFailed( const KRss::Feed::Id& id, const QString& errorMessage );
    void fetchAborted( const KRss::Feed::Id& id );

protected Q_SLOTS:

    void retrieveCollections();
    void retrieveItems( const Akonadi::Collection &col );
    bool retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts );

private Q_SLOTS:

    void initialize();
    void slotTagProviderRetrievalDone( KJob *job );
    void slotListingDone( KJob *job );
    void slotFeedCreationDone( KJob *job );
    void slotFeedModificationDone( KJob *job );
    void slotFeedDeletionDone( KJob *job );
    void slotBlockingFeedFetchDone( KJob *job );
    void slotParallelFeedFetchDone( KJob *job );

    void slotItemImportDone( KJob *job );
    void slotImportDone( KJob *job );
    void slotExportDone( KJob *job );

    void slotFetchProgress( KRss::FeedCollection::Id id, unsigned long percentage );

private:

    KRss::FeedCollection findFeed( const QString& xmlUrl ) const;

    void dump();

private:

    KRss::FeedCollection m_rootCollection;
    QHash<KRss::FeedCollection::Id, KRss::FeedCollection> m_feeds;
    QHash<KJob*, KRss::FeedCollection> m_feedsInProcess;
    QHash<KRss::FeedCollection::Id, FetchFeedJob*> m_fetchJobs;
    QHash<KJob*, QDBusMessage> m_replies;
};

#endif // KRSS_RSSRESOURCE_H
